/*
 * acompare.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <sax/FromXMLParserHelper.h>

#include "factory/XmlDataFactory.hpp"
#include "exception/CommonException.h"

#include "AutomatonCompare.h"
#include "GrammarCompare.h"

int main(int argc, char** argv) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("Regexp derivative compute binary", ' ', "0.01");

		TCLAP::UnlabeledValueArg<std::string> input1(	"input1",	"The first input to compare",	true,	"-",		"file");
		cmd.add( input1 );

		TCLAP::UnlabeledValueArg<std::string> input2(	"input2",	"The second input to compare",	true,	"-",		"file");
		cmd.add( input2 );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		std::deque<sax::Token> tokens1 = sax::FromXMLParserHelper::parseInput(input1);
		std::deque<sax::Token> tokens2 = sax::FromXMLParserHelper::parseInput(input2);

		int res;

		if(alib::XmlDataFactory::first<automaton::Automaton>(tokens1) && alib::XmlDataFactory::first<automaton::Automaton>(tokens2)) {
			automaton::Automaton automaton1 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(std::move(tokens1));
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(std::move(tokens2));

			measurements::end();
			measurements::start("Compare", measurements::Type::MAIN);

			res = AutomatonCompare::compare(automaton1, automaton2);
		} else if(alib::XmlDataFactory::first<grammar::Grammar>(tokens1) && alib::XmlDataFactory::first<grammar::Grammar>(tokens2)) {
			grammar::Grammar grammar1 = alib::XmlDataFactory::fromTokens<grammar::Grammar>(std::move(tokens1));
			grammar::Grammar grammar2 = alib::XmlDataFactory::fromTokens<grammar::Grammar>(std::move(tokens2));

			measurements::end();
			measurements::start("Compare", measurements::Type::MAIN);

			res = GrammarCompare::compare(grammar1, grammar2);
		} else {
			throw exception::CommonException("Only automata and grammars can be compared.");
		}

		measurements::end();
		measurements::end();

		return res;
	} catch (const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
