/*
 * BorderArrayNaive.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka, Jan Travnicek
 */

#include "BorderArrayNaive.h"
#include "SubtreeJumpTable.h"

#include <tree/ranked/PrefixRankedBarPattern.h>
#include <tree/ranked/PrefixRankedPattern.h>
#include <tree/Tree.h>
#include <global/GlobalData.h>

namespace arbology {

namespace exact {

std::vector < size_t > BorderArrayNaive::ba ( const tree::Tree & pattern ) {
	return dispatch ( pattern.getData ( ) );
}

bool BorderArrayNaive::matches ( const tree::PrefixRankedBarPattern & pattern, const std::vector < int > & subtreeJumpTable, int stop, int offset ) {
	unsigned i = 0;

	while ( offset < stop && i < pattern.getContent ( ).size ( ) )
		if ( pattern.getContent ( )[i] == pattern.getContent ( )[offset] ) {
			i++;
			offset++;
		} else if ( ( pattern.getContent ( )[i] == pattern.getSubtreeWildcard ( ) ) || ( pattern.getContent ( )[offset] == pattern.getSubtreeWildcard ( ) ) ) {
			i = subtreeJumpTable[i];
			offset = subtreeJumpTable[offset];
		} else {
			return false;
		}

	return true;
}

std::vector < size_t > BorderArrayNaive::ba ( const tree::PrefixRankedBarPattern & pattern ) {
	std::vector < int > patternSubtreeJumpTable = SubtreeJumpTable::compute ( pattern );
	std::vector < size_t > res;

	for ( unsigned i = 0; i <= pattern.getContent ( ).size ( ); i++ )
		res.push_back ( 0 );

	res[0] = -1;

	for ( unsigned i = 1; i <= pattern.getContent ( ).size ( ); i++ ) {
		int min = i;

		for ( unsigned j = 1; j < i; j++ )
			if ( matches ( pattern, patternSubtreeJumpTable, i, j ) ) {
				min = j;
				break;
			}

		res[i] = i - min;
	}

	if ( common::GlobalData::verbose )
		std::clog << res << std::endl;

	return res;
}

auto BorderArrayPrefixRankedBarPattern = BorderArrayNaive::RegistratorWrapper < std::vector < size_t >, tree::PrefixRankedBarPattern > ( BorderArrayNaive::ba );

bool BorderArrayNaive::matches ( const tree::PrefixRankedPattern & pattern, const std::vector < int > & subtreeJumpTable, int stop, int offset ) {
	unsigned i = 0;

	while ( offset < stop && i < pattern.getContent ( ).size ( ) )
		if ( pattern.getContent ( )[i] == pattern.getContent ( )[offset] ) {
			i++;
			offset++;
		} else if ( ( pattern.getContent ( )[i] == pattern.getSubtreeWildcard ( ) ) || ( pattern.getContent ( )[offset] == pattern.getSubtreeWildcard ( ) ) ) {
			i = subtreeJumpTable[i];
			offset = subtreeJumpTable[offset];
		} else {
			return false;
		}

	return true;
}

std::vector < size_t > BorderArrayNaive::ba ( const tree::PrefixRankedPattern & pattern ) {
	std::vector < int > patternSubtreeJumpTable = SubtreeJumpTable::compute ( pattern );
	std::vector < size_t > res;

	for ( unsigned i = 0; i <= pattern.getContent ( ).size ( ); i++ )
		res.push_back ( 0 );

	res[0] = -1;

	for ( unsigned i = 1; i <= pattern.getContent ( ).size ( ); i++ ) {
		int min = i;

		for ( unsigned j = 1; j < i; j++ )
			if ( matches ( pattern, patternSubtreeJumpTable, i, j ) ) {
				min = j;
				break;
			}

		res[i] = i - min;
	}

	if ( common::GlobalData::verbose )
		std::clog << res << std::endl;

	return res;
}

auto BorderArrayPrefixRankedPattern = BorderArrayNaive::RegistratorWrapper < std::vector < size_t >, tree::PrefixRankedPattern > ( BorderArrayNaive::ba );

} /* namespace exact */

} /* namespace arbology */
