/*
 * ExactSubtreeRepeatsNaive.h
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#ifndef _ARBOLOGY_SUBTREE_REPEATS_NAIVE_H_
#define _ARBOLOGY_SUBTREE_REPEATS_NAIVE_H_

#include <tree/TreeFeatures.h>
#include <alphabet/SymbolFeatures.h>
#include <core/multipleDispatch.hpp>

#include <map>
#include <vector>

namespace arbology {

namespace exact {

/**
 * Simple computation of subtree repeats
 */
class ExactSubtreeRepeatsNaive : public std::SingleDispatch < ExactSubtreeRepeatsNaive, tree::Tree, tree::TreeBase > {
	static tree::RankedNode repeats ( const tree::RankedNode & node, std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > & data, int & minId );
	static alphabet::RankedSymbol repeatsPrefixRanked ( const std::vector < alphabet::RankedSymbol > & symbols, std::vector < alphabet::RankedSymbol > & res, std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > & data, int & minId, int & index );
	static alphabet::RankedSymbol repeatsPrefixRankedBar ( const std::vector < alphabet::RankedSymbol > & symbols, std::vector < alphabet::RankedSymbol > & res, std::map < std::pair < alphabet::RankedSymbol, std::vector < alphabet::RankedSymbol > >, int > & data, int & minId, int & index );

public:
	/**
	 * Compute a same shaped tree with nodes containing unique subtree ids.
	 * @return Tree of repeats
	 */
	static tree::Tree repeats ( const tree::Tree & pattern );

	/**
	 * Compute a same shaped tree with nodes containing unique subtree ids.
	 * @return Tree of repeats
	 */
	static tree::RankedTree repeats ( const tree::RankedTree & tree );
	static tree::PrefixRankedTree repeats ( const tree::PrefixRankedTree & tree );
	static tree::PrefixRankedBarTree repeats ( const tree::PrefixRankedBarTree & tree );

};

} /* namespace exact */

} /* namespace arbology */

#endif /* _ARBOLOGY_SUBTREE_REPEATS_NAIVE_H_ */
