/*
 * SubtreeJumpTable.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#include "SubtreeJumpTable.h"

#include <tree/ranked/PrefixRankedTree.h>
#include <tree/ranked/PrefixRankedPattern.h>
#include <tree/ranked/PrefixRankedBarTree.h>
#include <tree/ranked/PrefixRankedBarPattern.h>
#include <alphabet/RankedSymbol.h>

#include <map>

namespace arbology {

namespace exact {

std::vector < int > SubtreeJumpTable::compute ( const tree::RankedTreeWrapper & subject ) {
	return dispatch ( subject.getData ( ) );
}

std::vector < int > SubtreeJumpTable::compute ( const tree::PrefixRankedBarTree & subject ) {
	std::vector < int > res;

	buildDataPointersBar ( res, subject, 0 );

	return res;
}

auto SubtreeSizesPrefixRankedBarTree = SubtreeJumpTable::RegistratorWrapper < std::vector < int >, tree::PrefixRankedBarTree > ( SubtreeJumpTable::compute );

std::vector < int > SubtreeJumpTable::compute ( const tree::PrefixRankedBarPattern & pattern ) {
	std::vector < int > res;

	buildDataPointersBar ( res, pattern, 0 );

	return res;
}

auto SubtreeSizesPrefixRankedBarPattern = SubtreeJumpTable::RegistratorWrapper < std::vector < int >, tree::PrefixRankedBarPattern > ( SubtreeJumpTable::compute );

std::vector < int > SubtreeJumpTable::compute ( const tree::PrefixRankedTree & subject ) {
	std::vector < int > res;

	buildDataPointersPrefixRanked ( res, subject, 0 );

	return res;
}

auto SubtreeSizesPrefixRankedTree = SubtreeJumpTable::RegistratorWrapper < std::vector < int >, tree::PrefixRankedTree > ( SubtreeJumpTable::compute );

std::vector < int > SubtreeJumpTable::compute ( const tree::PrefixRankedPattern & pattern ) {
	std::vector < int > res;

	buildDataPointersPrefixRanked ( res, pattern, 0 );

	return res;
}

auto SubtreeSizesPrefixRankedPattern = SubtreeJumpTable::RegistratorWrapper < std::vector < int >, tree::PrefixRankedPattern > ( SubtreeJumpTable::compute );

/**
 * used to compute subtree jump table.
 * @param begin - index of a root node of a complete subtree to process
 * @return index, increased by one, of the last node in the subtree starting at index begin
 */
template < class T >
int SubtreeJumpTable::buildDataPointersBar ( std::vector < int > & res, const T & subject, int begin ) {
	res.push_back ( 0 );
	int index = begin + 1;

	if ( ! subject.getBars ( ).count ( subject.getContent ( )[begin] ) )
		for ( unsigned i = 0; i < subject.getContent ( )[begin].getRank ( ).getData ( ); i++ )
			index = buildDataPointersBar ( res, subject, index );

	index++;
	res[begin] = index;
	res.push_back ( begin - 1 );
	return index;
}

/**
 * used to compute subtree jump table.
 * @param begin - index of a root node of a complete subtree to process
 * @return index, increased by one, of the last node in the subtree starting at index begin
 */
template < class T >
int SubtreeJumpTable::buildDataPointersPrefixRanked ( std::vector < int > & res, const T & subject, int begin ) {
	for ( unsigned i = 0; i < subject.getContent ( ).size ( ); i++ )
		res.push_back ( 0 );

	return buildDataPointersPrefixRankedInternal ( res, subject, begin );
}

template < class T >
int SubtreeJumpTable::buildDataPointersPrefixRankedInternal ( std::vector < int > & res, const T & subject, int begin ) {
	int index = begin + 1;

	for ( unsigned i = 0; i < subject.getContent ( )[begin].getRank ( ).getData ( ); i++ )
		index = buildDataPointersPrefixRankedInternal ( res, subject, index );

	res[begin] = index;
	return index;
}

} /* namespace exact */

} /* namespace arbology */
