/*
 * NFADeterminizer.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#include "common/NFACommon.h"

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <deque>
#include <algorithm>

namespace automaton {

namespace determinize {

automaton::DFA Determinize::determinize ( const automaton::MultiInitialStateNFA & nfa ) {
	 // 1, 4
	automaton::State initialState ( createDFAState ( nfa.getInitialStates ( ) ) );
	automaton::DFA res ( initialState );

	res.setInputAlphabet ( nfa.getInputAlphabet ( ) );

	 // 2
	std::deque < automaton::State > todo;
	todo.push_back ( std::move ( initialState ) );

	do {
		 // 3a, c
		automaton::State state = std::move ( todo.front ( ) );
		todo.pop_front ( );

		 // 3b
		for ( const alphabet::Symbol & input : nfa.getInputAlphabet ( ) ) {
			std::set < automaton::State > targetNFAStates;

			for ( automaton::State nfaState : make_dsubset_iterator ( state ) ) {
				auto iter = nfa.getTransitions ( ).find ( std::make_pair ( std::move ( nfaState ), input ) );

				if ( iter != nfa.getTransitions ( ).end ( ) )
					targetNFAStates.insert ( iter->second.begin ( ), iter->second.end ( ) );
			}

			automaton::State dfaState = createDFAState ( std::move ( targetNFAStates ) );

			 // 4
			bool existed = !res.addState ( dfaState );

			if ( !existed ) todo.push_back ( std::move ( dfaState ) );

			 // 3b
			res.addTransition ( std::move ( state ), input, std::move ( dfaState ) );
		}
	} while ( !todo.empty ( ) );

	 // 5
	const std::set<automaton::State>& finalLabels = nfa.getFinalStates();
	for ( const automaton::State & dfaState : res.getStates ( ) ) {
		auto nfaStates = make_dsubset_iterator ( dfaState );

		if(!std::excludes(finalLabels.begin(), finalLabels.end(), nfaStates.begin(), nfaStates.end()))
			res.addFinalState ( dfaState );
	}

	return res;
}

auto DeterminizeMultiInitialStateNFA = Determinize::RegistratorWrapper < automaton::DFA, automaton::MultiInitialStateNFA > ( Determinize::determinize );

automaton::DFA Determinize::determinize ( const automaton::NFA & nfa ) {
	 // 1, 4
	automaton::State initialState ( createDFAState ( { nfa.getInitialState ( ) } ) );
	automaton::DFA res ( initialState );

	res.setInputAlphabet ( nfa.getInputAlphabet ( ) );

	 // 2
	std::deque < automaton::State > todo;
	todo.push_back ( std::move ( initialState ) );

	do {
		 // 3a, c
		automaton::State state = std::move ( todo.front ( ) );
		todo.pop_front ( );

		 // 3b
		for ( const alphabet::Symbol & input : nfa.getInputAlphabet ( ) ) {
			std::set < automaton::State > targetNFAStates;

			for ( automaton::State nfaState : make_dsubset_iterator ( state ) ) {
				auto iter = nfa.getTransitions ( ).find ( std::make_pair ( std::move ( nfaState ), input ) );

				if ( iter != nfa.getTransitions ( ).end ( ) )
					targetNFAStates.insert ( iter->second.begin ( ), iter->second.end ( ) );
			}

			automaton::State dfaState = createDFAState ( std::move ( targetNFAStates ) );

			 // 4
			bool existed = !res.addState ( dfaState );

			if ( !existed ) todo.push_back ( dfaState );

			 // 3b
			res.addTransition ( std::move ( state ), input, std::move ( dfaState ) );
		}
	} while ( !todo.empty ( ) );

	 // 5
	const std::set<automaton::State>& finalLabels = nfa.getFinalStates();
	for ( const automaton::State & dfaState : res.getStates ( ) ) {
		auto nfaStates = make_dsubset_iterator ( dfaState );

		if(!std::excludes(finalLabels.begin(), finalLabels.end(), nfaStates.begin(), nfaStates.end()))
			res.addFinalState ( dfaState );
	}

	return res;
}

auto DeterminizeNFA = Determinize::RegistratorWrapper < automaton::DFA, automaton::NFA > ( Determinize::determinize );

} /* namespace determinize */

} /* namespace automaton */
