/*
 * Accept.cpp
 *
 *  Created on: 9. 2. 2014
 *	  Author: Jan Travnicek
 */

#include "Accept.h"
#include "Run.h"
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/TA/NFTA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/NPDTA.h>

#include <deque>
#include <vector>
#include <algorithm>

namespace automaton {

namespace run {

bool Accept::accept ( const automaton::Automaton & automaton, const alib::Object & object ) {
	return dispatch ( automaton.getData ( ), object.getData ( ) );
}

bool Accept::accept ( const automaton::Automaton & automaton, const string::LinearString & string ) {
	return dispatch ( automaton.getData ( ), string );
}

bool Accept::accept ( const automaton::Automaton & automaton, const tree::RankedTree & tree ) {
	return dispatch ( automaton.getData ( ), tree );
}

bool Accept::accept ( const automaton::DFA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	return std::get < 0 > ( res ) && automaton.getFinalStates ( ).count ( std::get < 1 > ( res ) );
}

auto AcceptDFALinearString = Accept::RegistratorWrapper < bool, automaton::DFA, string::LinearString > ( Accept::accept );

bool Accept::accept ( const automaton::NFA & automaton, const string::LinearString & string ) {
	std::tuple < bool, std::set < automaton::State >, std::set < unsigned > > res = Run::calculateStates ( automaton, string );

	return std::get < 0 > ( res ) && std::any_of ( std::get < 1 > ( res ).begin ( ), std::get < 1 > ( res ).end ( ), [&] ( const automaton::State & state ) {
				return automaton.getFinalStates ( ).count ( state );
			} );
}

auto AcceptNFALinearString = Accept::RegistratorWrapper < bool, automaton::NFA, string::LinearString > ( Accept::accept );

bool Accept::accept ( const automaton::DFTA & automaton, const tree::RankedTree & tree ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, tree );

	return std::get < 0 > ( res ) && automaton.getFinalStates ( ).count ( std::get < 1 > ( res ) );
}

auto AcceptDFTARankedTree = Accept::RegistratorWrapper < bool, automaton::DFTA, tree::RankedTree > ( Accept::accept );

bool Accept::accept ( const automaton::NFTA & automaton, const tree::RankedTree & tree ) {
	std::tuple < bool, std::set < automaton::State >, std::set < unsigned > > res = Run::calculateStates ( automaton, tree );

	return std::get < 0 > ( res ) && std::any_of ( std::get < 1 > ( res ).begin ( ), std::get < 1 > ( res ).end ( ), [&] ( const automaton::State & state ) {
				return automaton.getFinalStates ( ).count ( state );
			} );
}

auto AcceptNFTARankedTree = Accept::RegistratorWrapper < bool, automaton::NFTA, tree::RankedTree > ( Accept::accept );

bool Accept::accept ( const automaton::InputDrivenDPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	return std::get < 0 > ( res ) && ( automaton.getFinalStates ( ).count ( std::get < 1 > ( res ) ) || ( automaton.getFinalStates ( ).size ( ) == 0 && std::get < 3 > ( res ).size ( ) == 0 ) );
}

auto AcceptInputDrivenDPDALinearString = Accept::RegistratorWrapper < bool, automaton::InputDrivenDPDA, string::LinearString > ( Accept::accept );

bool Accept::accept ( const automaton::VisiblyPushdownDPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	return std::get < 0 > ( res ) && automaton.getFinalStates ( ).count ( std::get < 1 > ( res ) );
}

auto AcceptVisiblyPushdownDPDALinearString = Accept::RegistratorWrapper < bool, automaton::VisiblyPushdownDPDA, string::LinearString > ( Accept::accept );

bool Accept::accept ( const automaton::RealTimeHeightDeterministicDPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	return std::get < 0 > ( res ) && automaton.getFinalStates ( ).count ( std::get < 1 > ( res ) );
}

auto AcceptRealTimeHeightDeterministicDPDALinearString = Accept::RegistratorWrapper < bool, automaton::RealTimeHeightDeterministicDPDA, string::LinearString > ( Accept::accept );

bool Accept::accept ( const automaton::DPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	return std::get < 0 > ( res ) && ( automaton.getFinalStates ( ).count ( std::get < 1 > ( res ) ) || ( automaton.getFinalStates ( ).size ( ) == 0 && std::get < 3 > ( res ).size ( ) == 0 ) );
}

auto AcceptDPDALinearString = Accept::RegistratorWrapper < bool, automaton::DPDA, string::LinearString > ( Accept::accept );

bool Accept::accept ( const automaton::NPDTA & automaton, const string::LinearString & string ) {
	std::tuple < bool, std::set < automaton::State >, std::set < std::vector < alphabet::Symbol > > > res = Run::calculateStates ( automaton, string );
	return std::get<0>(res);
}

auto AcceptNPDTALinearString = Accept::RegistratorWrapper < bool, automaton::NPDTA, string::LinearString > ( Accept::accept );

} /* namespace run */

} /* namespace automaton */
