/*
 * AbsorbTerminalSymbol.cpp
 *
 *  Created on: 9. 6. 2015
 *	  Author: Jan Travnicek
 */

#include "AbsorbTerminalSymbol.h"

#include <grammar/ContextFree/CFG.h>
#include <alphabet/SymbolPairSymbol.h>

namespace grammar {

namespace parsing {

void AbsorbTerminalSymbol::handleAbsobtion ( const grammar::CFG & orig, grammar::CFG & res, const alphabet::Symbol & terminal, const std::set < alphabet::Symbol > & nonterminals, const std::map < alphabet::Symbol, alphabet::Symbol > & nonterminalsPrimed ) {
	for ( const alphabet::Symbol & nonterminal : nonterminals )
		for ( const std::vector < alphabet::Symbol > & rhs : orig.getRules ( ).find ( nonterminal )->second ) {
			std::vector < alphabet::Symbol > newRHS;

			for ( std::vector < alphabet::Symbol >::const_iterator iter = rhs.begin ( ); iter != rhs.end ( ); ++iter ) {
				if ( nonterminals.count ( * iter ) && ( ( iter + 1 == rhs.end ( ) ) || ( terminal == * ( iter + 1 ) ) ) ) {
					newRHS.push_back ( nonterminalsPrimed.find ( * iter )->second );

					if ( iter + 1 != rhs.end ( ) ) ++iter;
				} else if ( iter + 1 == rhs.end ( ) ) {
					newRHS.push_back ( * iter );
					newRHS.push_back ( terminal );
				} else {
					newRHS.push_back ( * iter );
				}
			}

			if ( rhs.size ( ) == 0 )
				newRHS.push_back ( terminal );

			res.addRule ( nonterminalsPrimed.find ( nonterminal )->second, newRHS );
		}

}

void AbsorbTerminalSymbol::absorbTerminalSymbol ( grammar::CFG & grammar, const alphabet::Symbol & terminal, const std::set < alphabet::Symbol > & nonterminals ) {
	grammar::CFG res ( grammar.getInitialSymbol ( ) );

	res.setNonterminalAlphabet ( grammar.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( grammar.getTerminalAlphabet ( ) );

	std::map < alphabet::Symbol, alphabet::Symbol > nonterminalsPrimed; // terminal is fixed in particular calls

	for ( const alphabet::Symbol & nonterminal : nonterminals ) {
		alphabet::Symbol newSymbol = alphabet::createUniqueSymbol ( alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( nonterminal, terminal ) ) ), res.getTerminalAlphabet ( ), res.getNonterminalAlphabet ( ) );
		res.addNonterminalSymbol ( newSymbol );
		nonterminalsPrimed.insert ( std::make_pair ( nonterminal, newSymbol ) );
	}

	handleAbsobtion ( grammar, res, terminal, nonterminals, nonterminalsPrimed );

	for ( const std::pair < const alphabet::Symbol, std::set < std::vector < alphabet::Symbol > > > & rule : grammar.getRules ( ) ) {
		const alphabet::Symbol & lhs = rule.first;

		for ( const std::vector < alphabet::Symbol > & rhs : rule.second ) {
			std::vector < alphabet::Symbol > newRHS;

			for ( std::vector < alphabet::Symbol >::const_iterator iter = rhs.begin ( ); iter != rhs.end ( ); ++iter ) {
				if ( ( iter + 1 != rhs.end ( ) ) && nonterminals.count ( * iter ) && ( terminal == * ( iter + 1 ) ) ) {
					newRHS.push_back ( nonterminalsPrimed.find ( * iter )->second );
					++iter;
				} else {
					newRHS.push_back ( * iter );
				}
			}

			res.addRule ( lhs, newRHS );
		}
	}

	grammar = res;
}

} /* namespace parsing */

} /* namespace grammar */
