/*
 * RecursiveNonterminal.h
 *
 *  Created on: 16. 11. 2015
 *	  Author: Jan Travnicek
 */

#ifndef RECURSIVE_NONTERMINAL_H_
#define RECURSIVE_NONTERMINAL_H_

#include <core/multipleDispatch.hpp>

#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class RecursiveNonterminal : public std::SingleDispatchLastStaticParam < RecursiveNonterminal, bool, grammar::GrammarBase, const alphabet::Symbol & > {
public:
	static bool isNonterminalRecursive ( const grammar::Grammar & grammar, const alphabet::Symbol & nonterminal );

	/**
	 * Retrieves A \in { B : A->^+ B \alpha, where \alpha \in (NuT)* } for given grammar and nonterminal
	 *
	 * @param grammar grammar
	 * @param nonterminal nonterminal
	 * @return bool which denote whether the nonterminal is recursive in the grammar
	 */
	template < class T >
	static bool isNonterminalRecursive ( const T & grammar, const alphabet::Symbol & nonterminal );

};

} /* namespace properties */

} /* namespace grammar */

#endif /* RECURSIVE_NONTERMINAL_H_ */
