/*
 * RegExpEmpty.cpp
 *
 *  Created on: 19. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "RegExpEmpty.h"
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>

namespace regexp {

namespace properties {

bool RegExpEmpty::languageIsEmpty(const regexp::RegExp& regexp) {
	return dispatch(regexp.getData());
}

bool RegExpEmpty::languageIsEmpty(const regexp::FormalRegExpElement& regexp) {
	bool out;
	regexp.Accept((void*) &out, RegExpEmpty::REG_EXP_EMPTY);
	return out;
}

bool RegExpEmpty::languageIsEmpty(const regexp::FormalRegExp& regexp) {
	return languageIsEmpty(regexp.getRegExp());
}

auto RegExpEmptyFormalRegExp = RegExpEmpty::RegistratorWrapper<bool, regexp::FormalRegExp>( RegExpEmpty::languageIsEmpty);

bool RegExpEmpty::languageIsEmpty(const regexp::UnboundedRegExpElement& regexp) {
	bool out;
	regexp.Accept((void*) &out, RegExpEmpty::REG_EXP_EMPTY);
	return out;
}

bool RegExpEmpty::languageIsEmpty(const regexp::UnboundedRegExp& regexp) {
	return languageIsEmpty(regexp.getRegExp());
}

auto RegExpEmptyUnboundedRegExp = RegExpEmpty::RegistratorWrapper<bool, regexp::UnboundedRegExp>( RegExpEmpty::languageIsEmpty);

// ----------------------------------------------------------------------------

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpAlternation& alternation) const {
	bool &ret = *(bool*) data;

	for(const auto& element : alternation.getElements()) {
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(data, *this);
		if(! ret) {
			ret = false;
			return;
		}
	}
	ret = true;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpConcatenation& concatenation) const {
	bool &ret = *(bool*) data;

	for(const auto& element : concatenation.getElements()) {
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(data, *this);
		if(ret) {
			ret = true;
			return;
		}
	}
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpIteration&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpSymbol&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpEmpty&) const {
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEmpty::Visit(void* data, const regexp::UnboundedRegExpEpsilon&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

// ----------------------------------------------------------------------------

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpAlternation& alternation) const {
	bool &ret = *(bool*) data;

	static_cast<const regexp::FormalRegExpElement&>(alternation.getLeftElement()).Accept(data, *this);
	bool retLeft = ret;

	static_cast<const regexp::FormalRegExpElement&>(alternation.getRightElement()).Accept(data, *this);
	bool retRight = ret;

	ret = retLeft && retRight;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpConcatenation& concatenation) const {
	bool &ret = *(bool*) data;

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getLeftElement()).Accept(data, *this);
	bool retLeft = ret;

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getRightElement()).Accept(data, *this);
	bool retRight = ret;

	ret = retLeft || retRight;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpIteration&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpSymbol&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpEmpty&) const {
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEmpty::Visit(void* data, const regexp::FormalRegExpEpsilon&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

// ----------------------------------------------------------------------------

const RegExpEmpty RegExpEmpty::REG_EXP_EMPTY;

} /* namespace properties */

} /* namespace regexp */
