/*
 * ExactCompare.cpp
 *
 *  Created on: Oct 9, 2014
 *	  Author: Radomir Polach */

#include "ExactCompare.h"

#include "string/LinearString.h"
#include "string/CyclicString.h"
#include "string/Epsilon.h"

namespace string {

namespace naive {

int ExactCompare::compare ( const string::String & u, const string::String & v ) {
	return dispatch ( u.getData ( ), v.getData ( ) );
}

int ExactCompare::compare ( const string::Epsilon &, const string::Epsilon & ) {
	return 0;
}

auto ExactCompareEpsilon = ExactCompare::RegistratorWrapper < int, string::Epsilon > ( ExactCompare::compare );

int ExactCompare::compare ( const string::LinearString & u, const string::LinearString & v ) {
	int n = ( int ) u.getContent ( ).size ( ), m = ( int ) v.getContent ( ).size ( );
	int k = 0;

	while ( k < n && k < m && u.getContent ( )[k] == v.getContent ( )[k] ) k++;

	if ( ( k == m ) && ( k == n ) )
		return 0;
	else if ( k == m )
		return -1;
	else if ( k == n )
		return 1;
	else if ( u.getContent ( )[k] < v.getContent ( )[k] )
		return -1;
	else
		return 1;
}

auto ExactCompareLinearString = ExactCompare::RegistratorWrapper < int, string::LinearString > ( ExactCompare::compare );

int ExactCompare::compare ( const string::CyclicString & u, const string::CyclicString & v ) {
	int n = ( int ) u.getContent ( ).size ( ), m = ( int ) v.getContent ( ).size ( );
	int i = -1, j = -1, k;

	bool last = 0;

	while ( i < n - 1 && j < m - 1 ) {
		k = 1;

		while ( k <= n && u.getContent ( )[( i + k ) % n] == v.getContent ( )[( j + k ) % m] ) k++;

		if ( k > n ) return 0;

		last = u.getContent ( )[( i + k ) % n] > v.getContent ( )[( j + k ) % m];

		if ( last )
			i += k;
		else
			j += k;
	}

	return last ? 1 : -1;
}

auto ExactCompareCyclicString = ExactCompare::RegistratorWrapper < int, string::CyclicString > ( ExactCompare::compare );

} /* namespace naive */

} /* namespace string */
