/*
 * ReversedBoyerMooreHorspool.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#include "ReversedBoyerMooreHorspool.h"
#include "ReversedBadCharacterShiftTable.h"

#include <string/LinearString.h>
#include <alphabet/Symbol.h>

#include <map>

namespace stringology {

namespace exact {

std::set < unsigned > ReversedBoyerMooreHorspool::match ( const string::String & subject, const string::String & pattern ) {
	return dispatch ( subject.getData ( ), pattern.getData ( ) );
}

std::set < unsigned > ReversedBoyerMooreHorspool::match ( const string::LinearString & string, const string::LinearString & pattern ) {
	std::set < unsigned > occ;
	std::map < alphabet::Symbol, size_t > bcs = ReversedBadCharacterShiftTable::bcs ( pattern ); // NOTE: the subjects alphabet must be a subset or equal to the pattern

	int haystack_offset = string.getContent ( ).size ( ) - pattern.getContent ( ).size ( );

	while ( haystack_offset >= 0 ) {
		size_t i = 0;

		while ( i < pattern.getContent ( ).size ( ) && string.getContent ( )[haystack_offset + i] == pattern.getContent ( )[i] )
			i++;

		 // Yay, there is match!!!
		if ( i == pattern.getContent ( ).size ( ) ) occ.insert ( haystack_offset );

		haystack_offset -= bcs[string.getContent ( )[haystack_offset]];

		// std::cout << haystack_offset << std::endl;
	}

	return occ;
}

auto ReversedBoyerMooreHorpoolLinearStringLinearString = ReversedBoyerMooreHorspool::RegistratorWrapper < std::set < unsigned >, string::LinearString, string::LinearString > ( ReversedBoyerMooreHorspool::match );

} /* namespace exact */

} /* namespace stringology */
