#include "CornerSubstitution.h"

#include "grammar/ContextFree/CFG.h"
#include "grammar/parsing/CornerSubstitution.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( CornerSubstitution, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( CornerSubstitution );

void CornerSubstitution::setUp ( ) {
}

void CornerSubstitution::tearDown ( ) {
}

void CornerSubstitution::testCornerSubstitution ( ) {
	alphabet::Symbol A = alphabet::symbolFrom ( "A" );
	alphabet::Symbol B = alphabet::symbolFrom ( "B" );
	alphabet::Symbol C = alphabet::symbolFrom ( "C" );

	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );
	alphabet::Symbol d = alphabet::symbolFrom ( 'd' );

	grammar::CFG grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < alphabet::Symbol > { a, B } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { C, B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { d } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { b, B } );

	grammar::CFG res = grammar;
	grammar::parsing::CornerSubstitution::cornerSubstitution ( res, a, A );

	grammar::CFG comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < alphabet::Symbol > { a, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, C, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { b, B, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { d } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { b, B } );

//	std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}

void CornerSubstitution::testCornerSubstitution2 ( ) {
	alphabet::Symbol A = alphabet::symbolFrom ( "A" );
	alphabet::Symbol B = alphabet::symbolFrom ( "B" );
	alphabet::Symbol C = alphabet::symbolFrom ( "C" );

	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );
	alphabet::Symbol d = alphabet::symbolFrom ( 'd' );

	grammar::CFG grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < alphabet::Symbol > { a, B } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { C, B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { a } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { B, b } );

	grammar::CFG res = grammar;
	grammar::parsing::CornerSubstitution::cornerSubstitution ( res, a, A );
	grammar::parsing::CornerSubstitution::cornerSubstitution ( res, a, A );

	grammar::CFG comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < alphabet::Symbol > { a, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, C, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, b, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { c, B, b, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { a } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { B, b } );

//	std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}
