#include "FollowTest.h"

#include "grammar/parsing/Follow.h"
#include "string/Epsilon.h"
#include "grammar/ContextFree/CFG.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( FollowTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( FollowTest );

void FollowTest::setUp ( ) {
}

void FollowTest::tearDown ( ) {
}

void FollowTest::testFollow ( ) {
	{
		alphabet::Symbol nE = alphabet::symbolFrom ( 'E' );
		alphabet::Symbol nT = alphabet::symbolFrom ( 'T' );
		alphabet::Symbol nF = alphabet::symbolFrom ( 'F' );

		alphabet::Symbol tP = alphabet::symbolFrom ( '+' );
		alphabet::Symbol tS = alphabet::symbolFrom ( '*' );
		alphabet::Symbol tL = alphabet::symbolFrom ( '(' );
		alphabet::Symbol tR = alphabet::symbolFrom ( ')' );
		alphabet::Symbol tA = alphabet::symbolFrom ( 'a' );

		grammar::CFG grammar ( nE );
		grammar.setTerminalAlphabet ( std::set < alphabet::Symbol > { tP, tS, tL, tR, tA } );
		grammar.setNonterminalAlphabet ( std::set < alphabet::Symbol > { nE, nT, nF } );

		std::vector < alphabet::Symbol > rhsE1 ( { nE, tP, nT } );
		std::vector < alphabet::Symbol > rhsE2 ( { nT } );
		std::vector < alphabet::Symbol > rhsT1 ( { nT, tS, nF } );
		std::vector < alphabet::Symbol > rhsT2 ( { nF } );
		std::vector < alphabet::Symbol > rhsF1 ( { tA } );
		std::vector < alphabet::Symbol > rhsF2 ( { tL, nE, tR } );

		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nE, rhsE2 );
		grammar.addRule ( nT, rhsT1 );
		grammar.addRule ( nT, rhsT2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < alphabet::Symbol, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > follow;

		follow[nE] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON, tP, tR
		};
		follow[nT] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON, tP, tR, tS
		};
		follow[nF] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON, tP, tR, tS
		};

		// --------------------------------------------------

		std::map < alphabet::Symbol, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > followAlgo;

		for ( const auto & nt : grammar.getNonterminalAlphabet ( ) )
			followAlgo[nt] = grammar::parsing::Follow::follow ( grammar, nt );

		 // std::cout << follow << std::endl;
		 // std::cout << followAlgo << std::endl;
		CPPUNIT_ASSERT ( follow == followAlgo );
	}

	{
		alphabet::Symbol nS = alphabet::symbolFrom ( 'S' );
		alphabet::Symbol nA = alphabet::symbolFrom ( 'A' );
		alphabet::Symbol nB = alphabet::symbolFrom ( 'B' );
		alphabet::Symbol nC = alphabet::symbolFrom ( 'C' );
		alphabet::Symbol nD = alphabet::symbolFrom ( 'D' );
		alphabet::Symbol nE = alphabet::symbolFrom ( 'E' );
		alphabet::Symbol nF = alphabet::symbolFrom ( 'F' );

		alphabet::Symbol tA = alphabet::symbolFrom ( 'a' );
		alphabet::Symbol tB = alphabet::symbolFrom ( 'b' );
		alphabet::Symbol tC = alphabet::symbolFrom ( 'c' );
		alphabet::Symbol tD = alphabet::symbolFrom ( 'd' );
		alphabet::Symbol tE = alphabet::symbolFrom ( 'e' );

		grammar::CFG grammar ( nS );
		grammar.setTerminalAlphabet ( std::set < alphabet::Symbol > { tA, tB, tC, tD, tE } );
		grammar.setNonterminalAlphabet ( std::set < alphabet::Symbol > { nS, nA, nB, nC, nD, nE, nF } );

		std::vector < alphabet::Symbol > rhsS1 ( { nB, tD, nS } );
		std::vector < alphabet::Symbol > rhsS2 ( { tD, tD, nC } );
		std::vector < alphabet::Symbol > rhsS3 ( { tC, nA } );
		std::vector < alphabet::Symbol > rhsA1 ( { tA, tE, nE } );
		std::vector < alphabet::Symbol > rhsA2 ( { tB, tB, nE } );
		std::vector < alphabet::Symbol > rhsB1 ( { tA, nF } );
		std::vector < alphabet::Symbol > rhsB2 ( { tB, tB, nD } );
		std::vector < alphabet::Symbol > rhsC1 ( { tA, nB, tD } );
		std::vector < alphabet::Symbol > rhsC2 ( { tE, nA } );
		std::vector < alphabet::Symbol > rhsD1 ( { tC, tA, nF } );
		std::vector < alphabet::Symbol > rhsE1 ( { tC, tA, tE, nE } );
		std::vector < alphabet::Symbol > rhsE2 ( { } );
		std::vector < alphabet::Symbol > rhsF1 ( { tE, nD } );
		std::vector < alphabet::Symbol > rhsF2 ( { } );

		grammar.addRule ( nS, rhsS1 );
		grammar.addRule ( nS, rhsS2 );
		grammar.addRule ( nS, rhsS3 );
		grammar.addRule ( nA, rhsA1 );
		grammar.addRule ( nA, rhsA2 );
		grammar.addRule ( nB, rhsB1 );
		grammar.addRule ( nB, rhsB2 );
		grammar.addRule ( nC, rhsC1 );
		grammar.addRule ( nC, rhsC2 );
		grammar.addRule ( nD, rhsD1 );
		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nE, rhsE2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < alphabet::Symbol, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > follow;

		follow[nS] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON
		};
		follow[nA] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON
		};
		follow[nB] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tD
		};
		follow[nC] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON
		};
		follow[nD] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tD
		};
		follow[nE] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON
		};
		follow[nF] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tD
		};

		// --------------------------------------------------

		std::map < alphabet::Symbol, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > followAlgo;

		for ( const auto & nt : grammar.getNonterminalAlphabet ( ) )
			followAlgo[nt] = grammar::parsing::Follow::follow ( grammar, nt );

		 // std::cout << follow << std::endl;
		 // std::cout << followAlgo << std::endl;
		CPPUNIT_ASSERT ( follow == followAlgo );
	}
}
