#include "LL1ParseTable.h"

#include "grammar/parsing/LL1ParseTable.h"
#include "string/Epsilon.h"
#include "grammar/ContextFree/CFG.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( LL1ParseTable, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( LL1ParseTable );

void LL1ParseTable::setUp ( ) {
}

void LL1ParseTable::tearDown ( ) {
}

void LL1ParseTable::testLL1Table ( ) {
	{
		alphabet::Symbol nE  = alphabet::symbolFrom ( 'E' );
		alphabet::Symbol nEp = alphabet::symbolFrom ( "E'" );
		alphabet::Symbol nT  = alphabet::symbolFrom ( 'T' );
		alphabet::Symbol nTp = alphabet::symbolFrom ( "T'" );
		alphabet::Symbol nF  = alphabet::symbolFrom ( 'F' );

		alphabet::Symbol tP = alphabet::symbolFrom ( '+' );
		alphabet::Symbol tS = alphabet::symbolFrom ( '*' );
		alphabet::Symbol tL = alphabet::symbolFrom ( '(' );
		alphabet::Symbol tR = alphabet::symbolFrom ( ')' );
		alphabet::Symbol tA = alphabet::symbolFrom ( 'a' );

		grammar::CFG grammar ( nE );
		grammar.setTerminalAlphabet ( std::set < alphabet::Symbol > { tP, tS, tL, tR, tA } );
		grammar.setNonterminalAlphabet ( std::set < alphabet::Symbol > { nE, nEp, nT, nTp, nF } );

		std::vector < alphabet::Symbol > rhsE1 ( { nT, nEp } );
		std::vector < alphabet::Symbol > rhsEp1 ( { tP, nT, nEp } );
		std::vector < alphabet::Symbol > rhsEp2 ( { } );
		std::vector < alphabet::Symbol > rhsT1 ( { nF, nTp } );
		std::vector < alphabet::Symbol > rhsTp1 ( { tS, nF, nTp } );
		std::vector < alphabet::Symbol > rhsTp2 ( { } );
		std::vector < alphabet::Symbol > rhsF1 ( { tA } );
		std::vector < alphabet::Symbol > rhsF2 ( { tL, nE, tR } );

		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nEp, rhsEp1 );
		grammar.addRule ( nEp, rhsEp2 );
		grammar.addRule ( nT, rhsT1 );
		grammar.addRule ( nTp, rhsTp1 );
		grammar.addRule ( nTp, rhsTp2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < std::pair < std::variant < alphabet::Symbol, string::Epsilon >, alphabet::Symbol >, std::set < std::vector < alphabet::Symbol > > > parseTable;

		parseTable[std::make_pair ( tA, nE )].insert ( rhsE1 );
		parseTable[std::make_pair ( tL, nE )].insert ( rhsE1 );

		parseTable[std::make_pair ( tP, nEp )].insert ( rhsEp1 );
		parseTable[std::make_pair ( tR, nEp )].insert ( rhsEp2 );
		parseTable[std::make_pair ( string::Epsilon::EPSILON, nEp )].insert ( rhsEp2 );

		parseTable[std::make_pair ( tA, nT )].insert ( rhsT1 );
		parseTable[std::make_pair ( tL, nT )].insert ( rhsT1 );

		parseTable[std::make_pair ( tS, nTp )].insert ( rhsTp1 );
		parseTable[std::make_pair ( tP, nTp )].insert ( rhsTp2 );
		parseTable[std::make_pair ( tR, nTp )].insert ( rhsTp2 );
		parseTable[std::make_pair ( string::Epsilon::EPSILON, nTp )].insert ( rhsTp2 );

		parseTable[std::make_pair ( tA, nF )].insert ( rhsF1 );
		parseTable[std::make_pair ( tL, nF )].insert ( rhsF2 );

		// --------------------------------------------------

		std::map < std::pair < std::variant < alphabet::Symbol, string::Epsilon >, alphabet::Symbol >, std::set < std::vector < alphabet::Symbol > > > parseTableAlgo = grammar::parsing::LL1ParseTable::parseTable ( grammar );

		std::cout << parseTable << std::endl;
		std::cout << parseTableAlgo << std::endl;
		CPPUNIT_ASSERT ( parseTable == parseTableAlgo );
	}
}
