#include "HopcroftTest.h"

#include <algorithm>

#include "graph/isomorphism/HopcroftImpl.h"
#include "graph/embedding/HopcroftTarjan.h"
#include "graph/generate/RandomGraphFactory.h"

CPPUNIT_TEST_SUITE_REGISTRATION(GraphHopcroftTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(GraphHopcroftTest, "graph");

static graph::AdjacencyListUndirectedGraph getGraph1()
{
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");

	graph::AdjacencyListUndirectedGraph ug1;
	ug1.addNode(n1);
	ug1.addNode(n2);
	ug1.addNode(n3);
	ug1.addNode(n4);
	ug1.addNode(n5);
	ug1.addEdge(graph::UndirectedEdge(n1, n2));
	ug1.addEdge(graph::UndirectedEdge(n1, n3));
	ug1.addEdge(graph::UndirectedEdge(n1, n4));
	ug1.addEdge(graph::UndirectedEdge(n2, n3));
	ug1.addEdge(graph::UndirectedEdge(n3, n4));
	ug1.addEdge(graph::UndirectedEdge(n5, n4));
	ug1.addEdge(graph::UndirectedEdge(n5, n3));
	ug1.addEdge(graph::UndirectedEdge(n5, n2));
	ug1.setEmbedding(graph::embedding::HopcroftTarjan::hopcrofttarjan(ug1));

	return ug1;
}

static graph::AdjacencyListUndirectedGraph getGraph1_I()
{
	graph::Node n1("ni1i");
	graph::Node n2("ni2i");
	graph::Node n3("ni3i");
	graph::Node n4("ni4i");
	graph::Node n5("ni5i");

	graph::AdjacencyListUndirectedGraph ug1;
	ug1.addNode(n1);
	ug1.addNode(n2);
	ug1.addNode(n3);
	ug1.addNode(n4);
	ug1.addNode(n5);
	ug1.addEdge(graph::UndirectedEdge(n2, n1));
	ug1.addEdge(graph::UndirectedEdge(n3, n2));
	ug1.addEdge(graph::UndirectedEdge(n4, n1));
	ug1.addEdge(graph::UndirectedEdge(n5, n4));
	ug1.addEdge(graph::UndirectedEdge(n3, n5));
	ug1.addEdge(graph::UndirectedEdge(n3, n1));
	ug1.addEdge(graph::UndirectedEdge(n5, n2));
	ug1.addEdge(graph::UndirectedEdge(n4, n3));
	ug1.setEmbedding(graph::embedding::HopcroftTarjan::hopcrofttarjan(ug1));

	return ug1;
}

static graph::AdjacencyListUndirectedGraph getGraph2()
{
	graph::Node n1("ni1");
	graph::Node n2("ni2");
	graph::Node n3("ni3");
	graph::Node n4("ni4");
	graph::Node n5("ni5");

	graph::AdjacencyListUndirectedGraph ug1;
	ug1.addNode(n1);
	ug1.addNode(n2);
	ug1.addNode(n3);
	ug1.addNode(n4);
	ug1.addNode(n5);
	ug1.addEdge(graph::UndirectedEdge(n1, n2));
	ug1.addEdge(graph::UndirectedEdge(n1, n2, "multi-edge1"));
	ug1.addEdge(graph::UndirectedEdge(n1, n2, "multi-edge2"));
	ug1.addEdge(graph::UndirectedEdge(n1, n2, "multi-edge3"));
	ug1.addEdge(graph::UndirectedEdge(n1, n3));
	ug1.addEdge(graph::UndirectedEdge(n1, n4));
	ug1.addEdge(graph::UndirectedEdge(n1, n4, "multi-edge4"));
	ug1.addEdge(graph::UndirectedEdge(n2, n3));
	ug1.addEdge(graph::UndirectedEdge(n2, n3, "multi-edge5"));
	ug1.addEdge(graph::UndirectedEdge(n3, n4));
	ug1.addEdge(graph::UndirectedEdge(n5, n4));
	ug1.addEdge(graph::UndirectedEdge(n5, n4, "multi-edge6"));
	ug1.addEdge(graph::UndirectedEdge(n5, n3));
	ug1.addEdge(graph::UndirectedEdge(n5, n2));

	std::unordered_map<graph::Node, std::vector<graph::Node>> embedding;
	embedding[n1] = { n4, n2, n2, n2, n2, n3, n4 };
	embedding[n2] = { n1, n1, n5, n3, n3, n1, n1 };
	embedding[n3] = { n4, n1, n2, n2, n5 };
	embedding[n4] = { n1, n1, n3, n5, n5 };
	embedding[n5] = { n3, n2, n4, n4 };
	ug1.setEmbedding(embedding);

	return ug1;
}

static graph::AdjacencyListUndirectedGraph getGraph2_I()
{
	graph::Node n1("n1i");
	graph::Node n2("n2i");
	graph::Node n3("n3i");
	graph::Node n4("n4i");
	graph::Node n5("n5i");

	graph::AdjacencyListUndirectedGraph ug1;
	ug1.addNode(n1);
	ug1.addNode(n2);
	ug1.addNode(n3);
	ug1.addNode(n4);
	ug1.addNode(n5);
	ug1.addEdge(graph::UndirectedEdge(n1, n2));
	ug1.addEdge(graph::UndirectedEdge(n2, n1, "multi-edge3"));
	ug1.addEdge(graph::UndirectedEdge(n1, n3));
	ug1.addEdge(graph::UndirectedEdge(n1, n4));
	ug1.addEdge(graph::UndirectedEdge(n4, n1, "multi-edge4"));
	ug1.addEdge(graph::UndirectedEdge(n2, n3));
	ug1.addEdge(graph::UndirectedEdge(n2, n1, "multi-edge2"));
	ug1.addEdge(graph::UndirectedEdge(n2, n3, "multi-edge5"));
	ug1.addEdge(graph::UndirectedEdge(n3, n4));
	ug1.addEdge(graph::UndirectedEdge(n4, n5));
	ug1.addEdge(graph::UndirectedEdge(n5, n4, "multi-edge6"));
	ug1.addEdge(graph::UndirectedEdge(n5, n3));
	ug1.addEdge(graph::UndirectedEdge(n1, n2, "multi-edge1"));
	ug1.addEdge(graph::UndirectedEdge(n5, n2));

	std::unordered_map<graph::Node, std::vector<graph::Node>> embedding;
	embedding[n1] = { n3, n4, n4, n2, n2, n2, n2 };
	embedding[n2] = { n5, n3, n3, n1, n1, n1, n1 };
	embedding[n3] = { n5, n4, n1, n2, n2 };
	embedding[n4] = { n3, n5, n5, n1, n1 };
	embedding[n5] = { n3, n2, n4, n4 };
	ug1.setEmbedding(embedding);

	return ug1;
}

void GraphHopcroftTest::testCircleIsomorphism()
{
	{
		std::vector<int> a = { 1, 2, 3, 4, 5 };
		std::vector<int> b = { 1, 2, 3, 4, 5 };
		graph::isomorphism::hopcroft::HopcroftImpl impl(nullptr, nullptr);
		CPPUNIT_ASSERT_EQUAL(true, impl.circleIsomorphism(a, b));
	}

	{
		std::vector<int> a = { 1, 2, 3, 4, 5 };
		std::vector<int> b = { 2, 3, 4, 5, 1 };
		graph::isomorphism::hopcroft::HopcroftImpl impl(nullptr, nullptr);
		CPPUNIT_ASSERT_EQUAL(true, impl.circleIsomorphism(a, b));
	}

	{
		std::vector<int> a = { 1, 2, 3, 4, 5 };
		std::vector<int> b = { 5, 1, 2, 3, 4 };
		graph::isomorphism::hopcroft::HopcroftImpl impl(nullptr, nullptr);
		CPPUNIT_ASSERT_EQUAL(true, impl.circleIsomorphism(a, b));
	}

	{
		std::vector<int> a = { 1, 2, 3, 4, 5 };
		std::vector<int> b = { 3, 4, 5, 1, 2 };
		graph::isomorphism::hopcroft::HopcroftImpl impl(nullptr, nullptr);
		CPPUNIT_ASSERT_EQUAL(true, impl.circleIsomorphism(a, b));
	}

	{
		std::vector<int> a = { 1, 2, 4, 4, 5 };
		std::vector<int> b = { 2, 3, 4, 5, 1 };
		graph::isomorphism::hopcroft::HopcroftImpl impl(nullptr, nullptr);
		CPPUNIT_ASSERT_EQUAL(false, impl.circleIsomorphism(a, b));
	}

	{
		std::vector<int> a = { 1, 2, 4, 5 };
		std::vector<int> b = { 2, 3, 4, 5, 1 };
		graph::isomorphism::hopcroft::HopcroftImpl impl(nullptr, nullptr);
		CPPUNIT_ASSERT_EQUAL(false, impl.circleIsomorphism(a, b));
	}
}

static const int MAX_STEPS = 1000;

#define RET_ERROR(info, expected, got) \
	{ \
		std::cout << info << " [ Expected:" << expected << " Got:" << got << " ]" << std::endl; \
		return false; \
	}

static graph::isomorphism::hopcroft::Vertex *findVertex(graph::isomorphism::hopcroft::Graph *g, const std::string &l)
{
	for (graph::isomorphism::hopcroft::Vertex *v : g->vertices)
		if (v->out == l)
			return v;

	CPPUNIT_ASSERT(false && "vertex not found");
	return nullptr;
}

static graph::isomorphism::hopcroft::EdgeEnd *findEdgeEnd(graph::isomorphism::hopcroft::Graph *g, const std::string &l)
{
	for (graph::isomorphism::hopcroft::EdgeEnd *e : g->edge_ends)
		if (l == e->otheredge->vertex->out + "->" + e->vertex->out)
			return e;

	CPPUNIT_ASSERT(false && "edge not found");
	return nullptr;
}

static bool checkEdge(graph::isomorphism::hopcroft::EdgeEnd *e, graph::isomorphism::hopcroft::Graph *g)
{
	if (e->isSpoke()) {

	}
	if (e->isLoopEdge()) {
		if (!e->otheredge->isLoopEdge())
			RET_ERROR("e->otheredge not loop edge", true, false);
	}
	if (e->isMultiEdge()) {
		if (!e->otheredge->isMultiEdge())
			RET_ERROR("e->otheredge not multi edge", true, false);
	}

	if (std::find(g->edge_ends.begin(), g->edge_ends.end(), e->cwedge) == g->edge_ends.end())
		RET_ERROR("cwedge not in graph", false, false);

	if (std::find(g->edge_ends.begin(), g->edge_ends.end(), e->ccwedge) == g->edge_ends.end())
		RET_ERROR("ccwedge not in graph", false, false);

	if (std::find(g->edge_ends.begin(), g->edge_ends.end(), e->otheredge) == g->edge_ends.end())
		RET_ERROR("otheredge not in graph", false, false);

	if (e->ccwface && std::find(g->faces.begin(), g->faces.end(), e->ccwface) == g->faces.end())
		RET_ERROR("ccwface not in graph", false, false);

	if (std::find(g->vertices.begin(), g->vertices.end(), e->vertex) == g->vertices.end())
		RET_ERROR("vertex not in graph", false, false);

	return true;
}

static bool checkIncidentEdges(graph::isomorphism::hopcroft::Vertex *v)
{
	int i = 0;
	std::set<graph::isomorphism::hopcroft::Vertex*> neighbors;
	v->forEachEdge([&i, &neighbors](graph::isomorphism::hopcroft::EdgeEnd *e) {
		neighbors.insert(e->vertex);
		return ++i <= MAX_STEPS;
	});

	if (i > MAX_STEPS)
		RET_ERROR("Over max_steps clockwise", false, false);

	if ((int)neighbors.size() != v->degree)
		RET_ERROR("Neighbors not equal to degree", neighbors.size(), v->degree);

	i = 0;
	v->forEachEdge([&i](graph::isomorphism::hopcroft::EdgeEnd *) {
		return ++i <= MAX_STEPS;
	});

	if (i > MAX_STEPS)
		RET_ERROR("Over max_steps counter-clockwise", false, false);

	return true;
}

static bool checkVEdge(graph::isomorphism::hopcroft::Vertex *v, graph::isomorphism::hopcroft::Graph *g)
{
	if (v->degree == 0 && !v->vedge)
		return true;

	if (std::find(g->edge_ends.begin(), g->edge_ends.end(), v->vedge) == g->edge_ends.end())
		RET_ERROR("vedge not in graph", false, false);

	int i = 0;
	graph::isomorphism::hopcroft::EdgeEnd *e = v->vedge;
	while (i == 0 || e != v->vedge) {
		if (++i > MAX_STEPS)
			RET_ERROR("Over max_steps", false, false);
		e = e->ccwedge;
	}

	return true;
}

static bool checkFace(graph::isomorphism::hopcroft::Face *f, graph::isomorphism::hopcroft::Graph *g)
{
	if (g->faces.size() == 1 && g->edge_ends.empty())
		return true;

	int i = 0;
	graph::isomorphism::hopcroft::EdgeEnd *e = f->fedge;

	while (i == 0 || e != f->fedge) {
		if (++i > MAX_STEPS)
			RET_ERROR("Over max_steps", false, false);
		if (e->ccwface != f)
			RET_ERROR("e->ccwface not f", f, e->ccwface);
		e = e->ccwedge;
	}

	if (i != f->edges)
		RET_ERROR("incorrect f->edges", i, f->edges);

	if (std::find(g->edge_ends.begin(), g->edge_ends.end(), f->fedge) == g->edge_ends.end())
		RET_ERROR("fedge not in graph", false, false);

	return true;
}

static void checkVisitAllEdges(graph::isomorphism::hopcroft::EdgeEnd *e, graph::isomorphism::hopcroft::Graph *g, std::set<graph::isomorphism::hopcroft::EdgeEnd*> &edges, std::set<graph::isomorphism::hopcroft::Vertex*> &vertices)
{
	if (edges.find(e) != edges.end())
		return;

	edges.insert(e);
	vertices.insert(e->vertex);

	checkEdge(e, g);
	checkVisitAllEdges(e->otheredge, g, edges, vertices);
	checkVisitAllEdges(e->cwedge, g, edges, vertices);
	checkVisitAllEdges(e->ccwedge, g, edges, vertices);
}

static bool checkGraphConsistency(graph::isomorphism::hopcroft::Graph *g)
{
	for (graph::isomorphism::hopcroft::Vertex *v : g->vertices) {
		if (!checkVEdge(v, g)) {
			std::cout << "FAIL: checkVEdge: " << v << std::endl;
			return false;
		}
		if (!checkIncidentEdges(v)) {
			std::cout << "FAIL: checkIncidentEdges: " << v << std::endl;
			return false;
		}
	}

	for (graph::isomorphism::hopcroft::EdgeEnd *e : g->edge_ends) {
		if (!checkEdge(e, g)) {
			std::cout << "FAIL: checkEdge: " << e << std::endl;
			return false;
		}
	}

	for (graph::isomorphism::hopcroft::Face *f : g->faces) {
		if (!checkFace(f, g)) {
			std::cout << "FAIL: checkFace: " << f << std::endl;
			return false;
		}
	}

	if (!g->edge_ends.empty()) {
		std::set<graph::isomorphism::hopcroft::EdgeEnd*> edges;
		std::set<graph::isomorphism::hopcroft::Vertex*> vertices;
		checkVisitAllEdges(g->edge_ends.front(), g, edges, vertices);
		if (edges.size() != g->edge_ends.size())
			return false;
	}
	return true;
}

void GraphHopcroftTest::testEmbedding()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();
	graph::isomorphism::hopcroft::Graph g1(ug1);
	CPPUNIT_ASSERT(checkGraphConsistency(&g1));

	graph::AdjacencyListUndirectedGraph ug2 = getGraph2();
	graph::isomorphism::hopcroft::Graph g2(ug2);
	CPPUNIT_ASSERT(checkGraphConsistency(&g2));
}

void GraphHopcroftTest::testEdges()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();
	graph::isomorphism::hopcroft::Graph g1(ug1);

	CPPUNIT_ASSERT(checkIncidentEdges(findVertex(&g1, "n1")));
	CPPUNIT_ASSERT(checkIncidentEdges(findVertex(&g1, "n2")));
	CPPUNIT_ASSERT(checkIncidentEdges(findVertex(&g1, "n3")));
	CPPUNIT_ASSERT(checkIncidentEdges(findVertex(&g1, "n4")));
	CPPUNIT_ASSERT(checkIncidentEdges(findVertex(&g1, "n5")));
}

void GraphHopcroftTest::testRemoveEdge()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();
	graph::AdjacencyListUndirectedGraph ug2 = getGraph2();

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);
		CPPUNIT_ASSERT_EQUAL(5, (int)g1.faces.size());
		CPPUNIT_ASSERT_EQUAL(5, (int)g1.vertices.size());
		CPPUNIT_ASSERT_EQUAL(16, (int)g1.edge_ends.size());
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));

		while (!g1.edge_ends.empty()) {
			g1.removeEdge(g1.edge_ends.front());
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
		}

		CPPUNIT_ASSERT_EQUAL(0, (int)g1.faces.size());
		CPPUNIT_ASSERT_EQUAL(0, (int)g1.vertices.size());
		for (graph::isomorphism::hopcroft::Vertex *v : g1.vertices)
			CPPUNIT_ASSERT(v->vedge == nullptr);
	}

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		while (!g1.edge_ends.empty()) {
			g1.removeEdge(g1.edge_ends.back());
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
		}

		CPPUNIT_ASSERT_EQUAL(0, (int)g1.faces.size());
		CPPUNIT_ASSERT_EQUAL(0, (int)g1.vertices.size());
		for (graph::isomorphism::hopcroft::Vertex *v : g1.vertices)
			CPPUNIT_ASSERT(v->vedge == nullptr);
	}

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		while (!g1.vertices.empty()) {
			graph::isomorphism::hopcroft::Vertex *v = g1.vertices.front();
			v->forEachEdge([&g1](graph::isomorphism::hopcroft::EdgeEnd *e) {
				g1.removeEdge(e, /* removeVertices */ false);
				CPPUNIT_ASSERT(checkGraphConsistency(&g1));
				return true;
			});

			CPPUNIT_ASSERT_EQUAL(0, v->degree);
			g1.removeVertex(v);
		}

		CPPUNIT_ASSERT_EQUAL(0, (int)g1.faces.size());
		CPPUNIT_ASSERT_EQUAL(0, (int)g1.edge_ends.size());
	}

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		while (!g1.vertices.empty()) {
			graph::isomorphism::hopcroft::Vertex *v = g1.vertices.back();
			v->forEachEdge([&g1](graph::isomorphism::hopcroft::EdgeEnd *e) {
				g1.removeEdge(e, /* removeVertices */ false);
				CPPUNIT_ASSERT(checkGraphConsistency(&g1));
				return true;
			});

			CPPUNIT_ASSERT_EQUAL(0, v->degree);
			g1.removeVertex(v);
		}

		CPPUNIT_ASSERT_EQUAL(0, (int)g1.faces.size());
		CPPUNIT_ASSERT_EQUAL(0, (int)g1.edge_ends.size());
	}

	{
		graph::isomorphism::hopcroft::Graph g2(ug2);

		while (!g2.vertices.empty()) {
			graph::isomorphism::hopcroft::Vertex *v = g2.vertices.back();
			v->forEachEdge([&g2](graph::isomorphism::hopcroft::EdgeEnd *e) {
				g2.removeEdge(e, /* removeVertices */ false);
				CPPUNIT_ASSERT(checkGraphConsistency(&g2));
				return true;
			});

			CPPUNIT_ASSERT_EQUAL(0, v->degree);
			g2.removeVertex(v);
		}

		CPPUNIT_ASSERT_EQUAL(0, (int)g2.faces.size());
		CPPUNIT_ASSERT_EQUAL(0, (int)g2.edge_ends.size());
	}
}

void GraphHopcroftTest::testAddInnerEdge()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		for (graph::isomorphism::hopcroft::Vertex *v : g1.vertices) {
			std::vector<graph::isomorphism::hopcroft::EdgeEnd*> edges;
			v->forEachEdge([&edges](graph::isomorphism::hopcroft::EdgeEnd *e) {
				edges.push_back(e);
				return true;
			});

			for (size_t i = 0; i < edges.size(); ++i) {
				auto *e1 = edges[i];
				auto *e2 = (i == edges.size() - 1) ? edges[0] : edges[i + 1];
				if (e1->vertex == e2->vertex)
					continue;

				g1.addInnerEdge(e1, e2);
				CPPUNIT_ASSERT(checkGraphConsistency(&g1));
			}
		}

		while (!g1.edge_ends.empty()) {
			g1.removeEdge(g1.edge_ends.back());
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
		}
	}

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		g1.removeEdge(findEdgeEnd(&g1, "n2->n5"));
		g1.removeEdge(findEdgeEnd(&g1, "n5->n4"));
		g1.removeEdge(findEdgeEnd(&g1, "n2->n1"));

		g1.addInnerEdge(findEdgeEnd(&g1, "n3->n2"), findEdgeEnd(&g1, "n3->n5"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));

		g1.removeEdge(findEdgeEnd(&g1, "n5->n2"));
		g1.removeEdge(findEdgeEnd(&g1, "n3->n4"));
		g1.removeEdge(findEdgeEnd(&g1, "n4->n1"));

		g1.addInnerEdge(findEdgeEnd(&g1, "n3->n2"), findEdgeEnd(&g1, "n3->n5"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));
	}
}

void GraphHopcroftTest::testRemoveDistinguishedEdge()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		g1.removeDistinguishedEdgeAndVertex(findEdgeEnd(&g1, "n3->n4"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));

		g1.removeEdge(findEdgeEnd(&g1, "n1->n4"));
		g1.removeEdge(findEdgeEnd(&g1, "n5->n4"));
		g1.removeEdge(findEdgeEnd(&g1, "n2->n5"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));

		g1.removeDistinguishedEdgeAndVertex(findEdgeEnd(&g1, "n2->n4"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));

		g1.removeEdge(findEdgeEnd(&g1, "n5->n4"));
		g1.removeEdge(findEdgeEnd(&g1, "n4->n1"));
		g1.removeEdge(findEdgeEnd(&g1, "n4->n1"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));
	}

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		g1.removeEdge(findEdgeEnd(&g1, "n5->n2"));
		g1.removeEdge(findEdgeEnd(&g1, "n2->n1"));
		g1.removeEdge(findEdgeEnd(&g1, "n2->n3"));
		g1.removeEdge(findEdgeEnd(&g1, "n5->n4"));
		g1.removeEdge(findEdgeEnd(&g1, "n1->n4"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));

		g1.removeDistinguishedEdgeAndVertex(findEdgeEnd(&g1, "n3->n4"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));
	}
}

void GraphHopcroftTest::testRemoveFourDegreeExceptionEdge()
{
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");

	graph::AdjacencyListUndirectedGraph ug1;
	ug1.addNode(n1);
	ug1.addNode(n2);
	ug1.addNode(n3);
	ug1.addNode(n4);
	ug1.addNode(n5);
	ug1.addEdge(graph::UndirectedEdge(n1, n2));
	ug1.addEdge(graph::UndirectedEdge(n1, n3));
	ug1.addEdge(graph::UndirectedEdge(n1, n4));
	ug1.addEdge(graph::UndirectedEdge(n2, n3));
	ug1.addEdge(graph::UndirectedEdge(n2, n4));
	ug1.addEdge(graph::UndirectedEdge(n3, n4));
	ug1.addEdge(graph::UndirectedEdge(n5, n4));
	ug1.addEdge(graph::UndirectedEdge(n5, n3));
	ug1.addEdge(graph::UndirectedEdge(n5, n2));
	ug1.setEmbedding(graph::embedding::HopcroftTarjan::hopcrofttarjan(ug1));

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		g1.removeFourDegreeExceptionVertex(findVertex(&g1, "n3"), findEdgeEnd(&g1, "n3->n2"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));
	}
}

void GraphHopcroftTest::testAddAndReconnectVertexIntoTriangularFace()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();

	{
		graph::isomorphism::hopcroft::Graph g1(ug1);

		g1.addAndReconnectVertexIntoTriangularFace(findEdgeEnd(&g1, "n3->n2"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));
		g1.addAndReconnectVertexIntoTriangularFace(findEdgeEnd(&g1, "n3->n4"));
		CPPUNIT_ASSERT(checkGraphConsistency(&g1));
		CPPUNIT_ASSERT_EQUAL(7, (int)g1.vertices.size());
	}
}

void GraphHopcroftTest::testIsomorphismByDefinition()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();
	graph::AdjacencyListUndirectedGraph ug1_i = getGraph1_I();
	graph::AdjacencyListUndirectedGraph ug2 = getGraph2();
	graph::AdjacencyListUndirectedGraph ug2_i = getGraph2_I();

	{
		// Same simple graphs
		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug1);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);
		CPPUNIT_ASSERT_EQUAL(true, impl.testByDefinition());
	}

	{
		// Same graphs with multi-edges and loops
		graph::isomorphism::hopcroft::Graph g1(ug2);
		graph::isomorphism::hopcroft::Graph g2(ug2);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);
		CPPUNIT_ASSERT_EQUAL(true, impl.testByDefinition());
	}

	{
		// Isomorphic simple graphs
		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug1_i);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);
		CPPUNIT_ASSERT_EQUAL(true, impl.testByDefinition());
	}

	{
		// Isomorphic graphs with multi-edges and loops
		graph::isomorphism::hopcroft::Graph g1(ug2);
		graph::isomorphism::hopcroft::Graph g2(ug2_i);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);
		CPPUNIT_ASSERT_EQUAL(true, impl.testByDefinition());
	}

	{
		// Different graphs
		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug2);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);
		CPPUNIT_ASSERT_EQUAL(false, impl.testByDefinition());
	}

	{
		// Different graphs
		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug2_i);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);
		CPPUNIT_ASSERT_EQUAL(false, impl.testByDefinition());
	}
}

void GraphHopcroftTest::testReductionsSameGraphs()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();
	graph::AdjacencyListUndirectedGraph ug2 = getGraph2();

	{
		// Same simple graphs
		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug1);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);
		while (impl.doReductions()) {
			CPPUNIT_ASSERT_EQUAL(false, impl.m_fail);
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
			CPPUNIT_ASSERT(checkGraphConsistency(&g2));
		}
		CPPUNIT_ASSERT_EQUAL(true, impl.testIsomorphism());
	}

	{
		// Same graphs with multi-edges and loops
		graph::isomorphism::hopcroft::Graph g1(ug2);
		graph::isomorphism::hopcroft::Graph g2(ug2);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);

		while (impl.doReductions()) {
			CPPUNIT_ASSERT_EQUAL(false, impl.m_fail);
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
			CPPUNIT_ASSERT(checkGraphConsistency(&g2));
		}
		CPPUNIT_ASSERT_EQUAL(true, impl.testIsomorphism());
	}
}

void GraphHopcroftTest::testReductionsIsomorphicGraphs()
{
	graph::AdjacencyListUndirectedGraph ug1 = getGraph1();
	graph::AdjacencyListUndirectedGraph ug1_i = getGraph1_I();
	graph::AdjacencyListUndirectedGraph ug2 = getGraph2();
	graph::AdjacencyListUndirectedGraph ug2_i = getGraph2_I();

	{
		// Isomorphic simple graphs
		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug1_i);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);
		while (impl.doReductions()) {
			CPPUNIT_ASSERT_EQUAL(false, impl.m_fail);
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
			CPPUNIT_ASSERT(checkGraphConsistency(&g2));
		}
		CPPUNIT_ASSERT_EQUAL(true, impl.testIsomorphism());
	}

	{
		// Isomorphic graphs with multi-edges and loops
		graph::isomorphism::hopcroft::Graph g1(ug2);
		graph::isomorphism::hopcroft::Graph g2(ug2_i);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);

		while (impl.doReductions()) {
			CPPUNIT_ASSERT_EQUAL(false, impl.m_fail);
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
			CPPUNIT_ASSERT(checkGraphConsistency(&g2));
		}
		CPPUNIT_ASSERT_EQUAL(true, impl.testIsomorphism());
	}
}

void GraphHopcroftTest::testReductionsOther()
{
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");
	graph::Node n6("n6");

	{
		graph::AdjacencyListUndirectedGraph ug1;
		ug1.addNode(n1);
		ug1.addNode(n2);
		ug1.addNode(n3);
		ug1.addNode(n4);
		ug1.addNode(n5);
		ug1.addNode(n6);
		ug1.addEdge(graph::UndirectedEdge(n1, n5));
		ug1.addEdge(graph::UndirectedEdge(n1, n6));
		ug1.addEdge(graph::UndirectedEdge(n1, n2));
		ug1.addEdge(graph::UndirectedEdge(n1, n3));
		ug1.addEdge(graph::UndirectedEdge(n2, n3));
		ug1.addEdge(graph::UndirectedEdge(n2, n4));
		ug1.addEdge(graph::UndirectedEdge(n2, n5));
		ug1.addEdge(graph::UndirectedEdge(n2, n6));
		ug1.addEdge(graph::UndirectedEdge(n3, n4));
		ug1.addEdge(graph::UndirectedEdge(n3, n5));
		ug1.addEdge(graph::UndirectedEdge(n4, n5));
		ug1.addEdge(graph::UndirectedEdge(n5, n6));
		ug1.setEmbedding(graph::embedding::HopcroftTarjan::hopcrofttarjan(ug1));

		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug1);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);

		while (impl.doReductions()) {
			CPPUNIT_ASSERT_EQUAL(false, impl.m_fail);
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
			CPPUNIT_ASSERT(checkGraphConsistency(&g2));
		}
		CPPUNIT_ASSERT_EQUAL(true, impl.testIsomorphism());
	}

	{
		// One node graph
		graph::AdjacencyListUndirectedGraph ug1;
		ug1.addNode(n1);

		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug1);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);

		while (impl.doReductions()) {
			CPPUNIT_ASSERT_EQUAL(false, impl.m_fail);
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
			CPPUNIT_ASSERT(checkGraphConsistency(&g2));
		}
		CPPUNIT_ASSERT_EQUAL(true, impl.testIsomorphism());

		findVertex(&g1, "n1")->vlabel = 1;
		findVertex(&g2, "n1")->vlabel = 2;
		CPPUNIT_ASSERT_EQUAL(false, impl.testIsomorphism());
	}

	{
		// Random
		graph::AdjacencyListUndirectedGraph ug1 = graph::generate::RandomGraphFactory::generateUndirectedTriConnectedPlanarGraph(30);
		ug1.setEmbedding(graph::embedding::HopcroftTarjan::hopcrofttarjan(ug1));
		graph::AdjacencyListUndirectedGraph ug2 = graph::generate::RandomGraphFactory::generateUndirectedIsomorphicGraph(ug1);
		ug2.setEmbedding(graph::embedding::HopcroftTarjan::hopcrofttarjan(ug2));

		graph::isomorphism::hopcroft::Graph g1(ug1);
		graph::isomorphism::hopcroft::Graph g2(ug2);
		graph::isomorphism::hopcroft::HopcroftImpl impl(&g1, &g2);

		while (impl.doReductions()) {
			if (impl.m_fail)
				break;
			CPPUNIT_ASSERT(checkGraphConsistency(&g1));
			CPPUNIT_ASSERT(checkGraphConsistency(&g2));
		}
		if (impl.m_fail || !impl.testIsomorphism())
			std::cout << "Random graph fail!" << std::endl;
	}
}
