#include "RegExpConcatenateTest.h"

#include "regexp/transform/RegExpConcatenate.h"

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExpElements.h>

#include <factory/StringDataFactory.hpp>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RegExpConcatenateTest, "regexp" );
CPPUNIT_TEST_SUITE_REGISTRATION( RegExpConcatenateTest );

void RegExpConcatenateTest::setUp() {
}

void RegExpConcatenateTest::tearDown() {
}

void RegExpConcatenateTest::testRegExpConcatenate() {
	{
		std::string input1 = "(#E a b)";
		regexp::RegExp re1 = alib::StringDataFactory::fromString<regexp::RegExp>(input1);

		std::string input2 = "(#E a c)";
		regexp::RegExp re2 = alib::StringDataFactory::fromString<regexp::RegExp>(input2);

		regexp::RegExp re = regexp::RegExpConcatenate::concatenate(re1, re2);

		std::string inputr = "(#E a b)(#E a c)";
		regexp::RegExp rer = alib::StringDataFactory::fromString<regexp::RegExp>(inputr);

		CPPUNIT_ASSERT(re == rer);
	}
	{
		std::string input1 = "(#E a b)";
		regexp::RegExp re1 = alib::StringDataFactory::fromString<regexp::RegExp>(input1);

		regexp::RegExp re2(regexp::FormalRegExp(regexp::FormalRegExpConcatenation(regexp::FormalRegExpEmpty {}, regexp::FormalRegExpEpsilon {})));

		regexp::RegExp re = regexp::RegExpConcatenate::concatenate(re1, re2);

		std::string inputr = "(#E a b)(#0 #E )";
		regexp::RegExp rer = alib::StringDataFactory::fromString<regexp::RegExp>(inputr);

		CPPUNIT_ASSERT(re == rer);
	}
	{
		regexp::RegExp re1(regexp::FormalRegExp(regexp::FormalRegExpConcatenation(regexp::FormalRegExpEmpty {}, regexp::FormalRegExpEpsilon {})));

		std::string input2 = "(#E a b)";
		regexp::RegExp re2 = alib::StringDataFactory::fromString<regexp::RegExp>(input2);

		regexp::RegExp re = regexp::RegExpConcatenate::concatenate(re1, re2);

		std::string inputr = "(#0 #E )(#E a b)";
		regexp::RegExp tmp = alib::StringDataFactory::fromString<regexp::RegExp>(inputr);
		regexp::RegExp rer(regexp::FormalRegExp(dynamic_cast<const regexp::UnboundedRegExp&>(tmp.getData())));

		std::cout << (dynamic_cast<const regexp::UnboundedRegExp&>(tmp.getData())).getAlphabet() << std::endl;
		std::cout << (dynamic_cast<const regexp::FormalRegExp&>(rer.getData())).getAlphabet() << std::endl;

		std::cout << re << std::endl;
		std::cout << rer << std::endl;
		CPPUNIT_ASSERT(re == rer);
	}
}
