#include "BackwardNondeterministicDAWGMatchingTest.h"

#include "string/String.h"
#include "stringology/exact/BackwardNondeterministicDAWGMatching.hpp"
#include "stringology/exact/ExactFactorMatch.h"

#include "string/generate/RandomStringFactory.h"
#include "string/generate/RandomSubstringFactory.h"

#define CPPUNIT_IMPLY( x, y )    CPPUNIT_ASSERT ( !( x ) || ( y ) )

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( BackwardNondeterministicDAWGMatchingTest, "stringology" );
CPPUNIT_TEST_SUITE_REGISTRATION ( BackwardNondeterministicDAWGMatchingTest );

void BackwardNondeterministicDAWGMatchingTest::setUp ( ) {
}

void BackwardNondeterministicDAWGMatchingTest::tearDown ( ) {
}

void BackwardNondeterministicDAWGMatchingTest::testBNDM ( ) {

    std::vector<std::string> subjects;
    std::vector<std::string> patterns;
    std::vector<std::set<unsigned>> expectedOccs;

    subjects.push_back("a"); patterns.push_back("a"); expectedOccs.push_back({0});
    subjects.push_back("a"); patterns.push_back("b"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfa"); patterns.push_back("alfalfalfa"); expectedOccs.push_back({0});
    subjects.push_back("alfalfalfa"); patterns.push_back("blfalfalfa"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfa"); patterns.push_back("alfalfalfb"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); expectedOccs.push_back({0});
    subjects.push_back("alfalfalfaalfalfalfaabfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfaa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfaa"); expectedOccs.push_back({0});
    subjects.push_back("atggccttgcc"); patterns.push_back("gcc"); expectedOccs.push_back({3,8});
    subjects.push_back("aaaaaaaaaa"); patterns.push_back("a"); expectedOccs.push_back({0,1,2,3,4,5,6,7,8,9});


    for(size_t i = 0; i < subjects.size(); ++i) {
        string::String subject = string::stringFrom ( subjects[i] );
        string::String pattern = string::stringFrom ( patterns[i] );
        std::set < unsigned > res = stringology::exact::BackwardNondeterministicDAWGMatching::match ( subject, pattern );
        CPPUNIT_ASSERT ( res == expectedOccs[i] );
        res = stringology::exact::BackwardNondeterministicDAWGMatching::match64 ( subject, pattern );
        CPPUNIT_ASSERT ( res == expectedOccs[i] );
        res = stringology::exact::BackwardNondeterministicDAWGMatching::match128 ( subject, pattern );
        CPPUNIT_ASSERT ( res == expectedOccs[i] );
        std::cout << subjects[i] << ' ' << patterns[i] << ' ' << res << std::endl;
    }

    auto longSubject = string::generate::RandomStringFactory::generateLinearString (64 * 64 * 64, 512, false, true);
    auto longPattern = string::generate::RandomSubstringFactory::generateSubstring(64 * 32 * 32, longSubject );
    std::set < unsigned > res = stringology::exact::BackwardNondeterministicDAWGMatching::match ( longSubject, longPattern );
    std::set < unsigned > resRef = stringology::exact::ExactFactorMatch::match ( longSubject, longPattern );
    std::cout << "long: " << res << std::endl;
    CPPUNIT_ASSERT ( res == resRef);
}
