/*
 * CommonException.h
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#ifndef COMMON_EXCEPTION_H_
#define COMMON_EXCEPTION_H_

#include <deque>
#include <exception>
#include <string>
#include <ostream>
#include "../sax/Token.h"

#include "../object/ObjectBase.h"

namespace exception {

/**
 * Basic exception from which are derived all other exceptions.
 * Contains reason why the exception occurred.
 */
class CommonException : public std::exception, public alib::ObjectBase {
protected:
	std::string cause;
	std::string backtrace;
	std::string whatMessage;
	std::string command;

public:
	explicit CommonException ( std::string cause );

	explicit CommonException ( std::string cause, std::string backtrace, std::string command );

	virtual alib::ObjectBase * clone ( ) const;

	virtual alib::ObjectBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const CommonException & other ) const;

	/**
	 * @return reason why the exception occurred
	 */
	const char * what ( ) const noexcept;

	/**
	 * @return reason why the exception occurred
	 */
	const std::string & getCause ( ) const;

	/**
	 * @return reason why the exception occurred
	 */
	const std::string & getBacktrace ( ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "CommonException";

		return xmlTagName;
	}

	static CommonException parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace exception */

namespace std {

template < >
struct compare < ::exception::CommonException > {
	int operator ()( const::exception::CommonException & first, const::exception::CommonException & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* COMMON_EXCEPTION_H_ */
