/*
 * Author: Radovan Cerveny
 */

#ifndef MEASUREMENT_RESULTS_XML_HPP_
#define MEASUREMENT_RESULTS_XML_HPP_

#include <deque>
#include "../sax/Token.h"
#include "MeasurementResults.hpp"

namespace measurements {

class MeasurementResultsXml {

	static std::string MEASUREMENT_RESULTS_TAG;

	static std::string MEASUREMENT_FRAME_TAG;
	static std::string MEASUREMENT_FRAME_NAME_TAG;
	static std::string MEASUREMENT_FRAME_TYPE_TAG;

	static std::string MEASUREMENT_SUBFRAMES_TAG;

	 // TimeDataFrame
	static std::string TIME_DATA_FRAME_TAG;
	static std::string TIME_DATA_FRAME_DURATION_TAG;
	static std::string TIME_DATA_FRAME_IN_FRAME_DURATION_TAG;

	 // MemoryDataFrame
	static std::string MEMORY_DATA_FRAME_TAG;
	static std::string MEMORY_DATA_FRAME_START_HEAP_USAGE_TAG;
	static std::string MEMORY_DATA_FRAME_END_HEAP_USAGE_TAG;
	static std::string MEMORY_DATA_FRAME_HIGH_WATERMARK_TAG;
	static std::string MEMORY_DATA_FRAME_IN_FRAME_HIGH_WATERMARK_TAG;

	 // CounterDataFrame
	static std::string COUNTER_DATA_FRAME_TAG;
	static std::string COUNTER_DATA_FRAME_COUNTERS_TAG;
	static std::string COUNTER_DATA_FRAME_IN_FRAME_COUNTERS_TAG;
	static std::string COUNTER_DATA_FRAME_COUNTER_TAG;
	static std::string COUNTER_DATA_FRAME_COUNTER_NAME_TAG;
	static std::string COUNTER_DATA_FRAME_COUNTER_VALUE_TAG;

	static void composeMeasurementFrames ( std::deque < sax::Token > &, unsigned, const stealth_vector < MeasurementFrame > & );
	static void composeTimeDataFrame ( std::deque < sax::Token > &, unsigned, const stealth_vector < MeasurementFrame > & );
	static void composeMemoryDataFrame ( std::deque < sax::Token > &, unsigned, const stealth_vector < MeasurementFrame > & );
	static void composeCounterDataFrame ( std::deque < sax::Token > &, unsigned, const stealth_vector < MeasurementFrame > & );

	static void parseRootMeasurementFrame ( MeasurementResults &, std::deque < sax::Token >::iterator & );
	static void parseSubframes ( unsigned, MeasurementResults &, std::deque < sax::Token >::iterator & );
	static void parseMeasurementFrame ( unsigned, MeasurementResults &, std::deque < sax::Token >::iterator & );
	static void parseTimeDataFrame ( MeasurementFrame &, std::deque < sax::Token >::iterator & );
	static void parseMemoryDataFrame ( MeasurementFrame &, std::deque < sax::Token >::iterator & );
	static void parseCounterDataFrame ( MeasurementFrame &, std::deque < sax::Token >::iterator & );
	static void parseCounterDataFrameCounters ( const std::string &, stealth_map < stealth_string, CounterHint::value_type > &, std::deque < sax::Token >::iterator & );

	template < typename T >
	static T valueTypeFromString ( const std::string & );

public:
	static std::deque < sax::Token > compose ( const MeasurementResults & );
	static void compose ( const MeasurementResults &, std::deque < sax::Token > & );

	static MeasurementResults parse ( std::deque < sax::Token > & );
	static MeasurementResults parse ( std::deque < sax::Token >::iterator & );
};

}

#endif /* MEASUREMENT_RESULTS_XML_HPP_ */
