/*
 * EndSymbol.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef END_SYMBOL_H_
#define END_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class EndSymbol : public SymbolBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit EndSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const EndSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static EndSymbol END;
	static Symbol END_SYMBOL;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "EndSymbol";

		return xmlTagName;
	}

	static EndSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* END_SYMBOL_H_ */
