/*
 * Symbol.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "Symbol.h"
#include "LabeledSymbol.h"
#include <climits>
#include "SymbolException.h"
#include <XmlApi.hpp>

namespace alphabet {

Symbol Symbol::next ( ) const {
	return Symbol ( this->getData ( ).next ( ) );
}

void Symbol::inc ( ) {
	SymbolBase * res = std::move ( this->getData ( ) ).inc ( );

	if ( res == NULL ) return;

	this->setData ( res );
}

alphabet::Symbol createUniqueSymbol ( alphabet::Symbol attempt, const std::set < alphabet::Symbol > & terminalAlphabet, const std::set < alphabet::Symbol > & nonterminalAlphabet ) {
	int i = 0;

	do {
		if ( ( terminalAlphabet.count ( attempt ) == 0 ) && ( nonterminalAlphabet.count ( attempt ) == 0 ) )
			return attempt;

		attempt.inc ( );
	} while ( i++ < INT_MAX );

	throw SymbolException ( "Could not create unique symbol." );
}

alphabet::Symbol symbolFrom ( int number ) {
	return alphabet::Symbol { alphabet::LabeledSymbol { label::labelFrom ( number ) } };
}

alphabet::Symbol symbolFrom ( char character ) {
	return alphabet::Symbol { alphabet::LabeledSymbol { label::labelFrom ( character ) } };
}

alphabet::Symbol symbolFrom ( std::string string ) {
	return alphabet::Symbol { alphabet::LabeledSymbol { label::labelFrom ( std::move ( string ) ) } };
}

alphabet::Symbol symbolFrom ( const char * string ) {
	return symbolFrom ( ( std::string ) string );
}

} /* namespace alphabet */

namespace alib {

auto SymbolDeleter = xmlApi < alphabet::Symbol >::InputContextDeleter ( );
}
