/*
 * SymbolSetSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_SET_SYMBOL_H_
#define SYMBOL_SET_SYMBOL_H_

#include "Symbol.h"
#include <set>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class SymbolSetSymbol : public SymbolBase {
protected:
	std::set < Symbol > symbol;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit SymbolSetSymbol ( std::set < Symbol > symbol );

	virtual SymbolBase * clone ( ) const;

	virtual SymbolBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const std::set < Symbol > & getData ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const SymbolSetSymbol & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "SymbolSetSymbol";

		return xmlTagName;
	}

	static SymbolSetSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* SYMBOL_SET_SYMBOL_H_ */
