/*
 * State.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#include "State.h"
#include "../../label/Label.h"
#include <XmlApi.hpp>

namespace automaton {

State::State(label::Label name) : name(std::move(name)) {
}

State::State(int number) : name(label::labelFrom(number)) {
}

State::State(int number1, int number2) : name(label::labelFrom(number1, number2)) {
}

State::State(char character) : name(label::labelFrom(character)) {
}

State::State(std::string name) : name(label::labelFrom(std::move(name))) {
}

State::State(const char* name) : State((std::string) name) {
}

const label::Label& State::getName() const {
	return name;
}

label::Label& State::getName() {
	return name;
}

bool State::operator < (const State& other) const {
	return this->compare(other) < 0;
}

bool State::operator == (const State& other) const {
	return this->compare(other) == 0;
}

bool State::operator != (const State& other) const{
	return !(*this == other);
}

int State::compare(const State& other) const {
	return name.getData().compare(other.name.getData());
}

std::ostream& operator<<(std::ostream& out, const State& state) {
	out << "(State " << state.name << ")";
	return out;
}

State::operator std::string () const {
	return (std::string) name;
}

State State::parse(std::deque<sax::Token>::iterator& input) {
	return State(alib::xmlApi<label::Label>::parse(input));
}

void State::compose(std::deque<sax::Token>& out) const {
	alib::xmlApi<label::Label>::compose(out, name);
}

} /* namespace automaton */

namespace alib {

automaton::State xmlApi<automaton::State>::parse(std::deque<sax::Token>::iterator& input) {
	return automaton::State::parse(input);
}

bool xmlApi<automaton::State>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, automaton::State::getXmlTagName());
}

void xmlApi<automaton::State>::compose(std::deque<sax::Token>& output, const automaton::State& data) {
	data.compose(output);
}

} /* namespace alib */
