/*
 * Pair.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "ObjectsPair.h"

#include <sstream>

#include "Container.h"
#include <object/Object.h>
#include <XmlApi.hpp>

namespace container {

ObjectsPair::ObjectsPair(std::pair<alib::Object, alib::Object> pair) : std::pair<alib::Object, alib::Object>(std::move(pair)) {

}

ObjectsPair::ObjectsPair(alib::Object first, alib::Object second) : std::pair<alib::Object, alib::Object>(std::move(first), std::move(second)) {

}

ContainerBase* ObjectsPair::clone() const {
	return new ObjectsPair(*this);
}

ContainerBase* ObjectsPair::plunder() && {
	return new ObjectsPair(std::move(*this));
}

int ObjectsPair::compare(const ObjectsPair& other) const {
	std::compare<std::pair<alib::Object, alib::Object>> comp;
	return comp(static_cast<const std::pair<alib::Object, alib::Object>>(*this), static_cast<const std::pair<alib::Object, alib::Object>>(other));
}

void ObjectsPair::operator>>(std::ostream& os) const {
	os << "(ObjectsPair " << static_cast<const std::pair<alib::Object, alib::Object>>(*this) << ")";
}

ObjectsPair::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return std::move(ss).str();
}

ObjectsPair ObjectsPair::parse(std::deque<sax::Token>::iterator& input) {
	return ObjectsPair(parseRaw<alib::Object, alib::Object>(input));
}

void ObjectsPair::compose(std::deque<sax::Token>& out) const {
	compose(out, *this);
}

} /* namespace container */

namespace alib {

auto objectsPairParserRegister = xmlApi<container::Container>::ParserRegister<container::ObjectsPair>();
auto objectsPairParserRegister2 = xmlApi<alib::Object>::ParserRegister<container::ObjectsPair>();

} /* namespace alib */
