/*
 * LR0ItemsLabel.h
 *
 *  Created on: 29. 4. 2016
 *      Author: Martin Kocicka
 */

#ifndef LR0ITEMS_LABEL_H_
#define LR0ITEMS_LABEL_H_

#include <map>
#include <set>
#include <vector>

#include <object/Object.h>
#include "LabelBase.h"
#include "Label.h"
#include "../alphabet/Symbol.h"
#include "../grammar/parsing/LRParserTypes.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class LR0ItemsLabel : public LabelBase {
protected:
	grammar::parsing::LR0Items items;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LR0ItemsLabel ( grammar::parsing::LR0Items items );

	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LR0ItemsLabel ( );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	const grammar::parsing::LR0Items & getItems ( ) const {
		return items;
	}

	grammar::parsing::LR0Items & getItems ( ) {
		return items;
	}

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const LR0ItemsLabel & other ) const;

	bool operator < (const LR0ItemsLabel& other) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "LR0ItemsLabel";

		return xmlTagName;
	}

	static LR0ItemsLabel parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual LabelBase * next ( ) const;
	virtual LabelBase * inc ( ) &&;
};

} /* namespace label */

namespace std {

template < >
struct compare < label::LR0ItemsLabel > {
	int operator ()( const label::LR0ItemsLabel & first, const label::LR0ItemsLabel & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* LR0ITEMS_LABEL_H_ */
