/*
 * Bool.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_BOOL_H_
#define PRIMITIVE_BOOL_H_

#include "PrimitiveBase.h"
#include <XmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Bool : public PrimitiveBase {
protected:
	bool data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Bool ( bool data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	bool getData ( ) const;

	/**
	 * sets new content of primitive
	 */
	void setData ( bool data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Bool & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Bool";

		return xmlTagName;
	}

	static Bool parse ( std::deque < sax::Token >::iterator & input );
	static bool parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	static void compose ( std::deque < sax::Token > & out, bool primitive );

	Bool next ( ) const;
	virtual void inc ( );
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < bool > {
	static bool parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, bool data );
};

} /* namespace alib */

#endif /* PRIMITIVE_BOOL_H_ */
