/*
 * FormalRegExp.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "FormalRegExp.h"
#include <exception/CommonException.h>
#include "FormalRegExpEmpty.h"

#include <iostream>
#include <algorithm>
#include <sstream>

#include "../unbounded/UnboundedRegExp.h"

#include <sax/FromXMLParserHelper.h>
#include "../common/RegExpFromXMLParser.h"
#include "../common/RegExpToXMLComposer.h"
#include "../RegExp.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include <cast/CastApi.hpp>

namespace regexp {

FormalRegExp::FormalRegExp ( std::set < alphabet::Symbol > alphabet, FormalRegExpElement && regExp ) : std::Components < FormalRegExp, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( alphabet ) ), std::tuple < > ( ) ), regExp ( NULL ) {
	setRegExp ( std::move ( regExp ) );
}

FormalRegExp::FormalRegExp ( std::set < alphabet::Symbol > alphabet, const FormalRegExpElement & regExp ) : FormalRegExp ( std::move ( alphabet ), std::move_copy ( regExp ) ) {
}

FormalRegExp::FormalRegExp ( ) : FormalRegExp ( std::set < alphabet::Symbol > ( ), FormalRegExpEmpty ( ) ) {
}

FormalRegExp::FormalRegExp ( FormalRegExpElement && regExp ) : FormalRegExp ( regExp.computeMinimalAlphabet ( ), regExp ) {
}

FormalRegExp::FormalRegExp ( const FormalRegExpElement & regExp ) : FormalRegExp ( regExp.computeMinimalAlphabet ( ), std::move_copy ( regExp ) ) {
}

FormalRegExp::FormalRegExp ( const UnboundedRegExp & other ) : FormalRegExp ( other.getAlphabet ( ), std::move ( * other.getRegExp ( ).asFormal ( ) ) ) {
}

RegExpBase * FormalRegExp::clone ( ) const {
	return new FormalRegExp ( * this );
}

RegExpBase * FormalRegExp::plunder ( ) && {
	return new FormalRegExp ( std::move ( * this ) );
}

const FormalRegExpElement & FormalRegExp::getRegExp ( ) const {
	return * regExp;
}

FormalRegExpElement & FormalRegExp::getRegExp ( ) {
	return * regExp;
}

void FormalRegExp::setRegExp ( const FormalRegExpElement & regExp ) {
	setRegExp ( std::move_copy ( regExp ) );
}

void FormalRegExp::setRegExp ( FormalRegExpElement && param ) {
	this->regExp = std::smart_ptr < FormalRegExpElement > ( std::move ( param ).plunder ( ) );

	if ( !regExp->checkAlphabet ( getAlphabet ( ) ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );
}

void FormalRegExp::operator >>( std::ostream & out ) const {
	out << "(FormalRegExp " << * ( this->regExp ) << ")";
}

int FormalRegExp::compare ( const FormalRegExp & other ) const {
	auto first = std::tie ( regExp, getAlphabet ( ) );
	auto second = std::tie ( other.regExp, other.getAlphabet ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

FormalRegExp::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

FormalRegExp FormalRegExp::parse ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, FormalRegExp::getXmlTagName() );

	std::set < alphabet::Symbol > alphabet = RegExpFromXMLParser::parseAlphabet ( input );

	std::rvalue_ref < FormalRegExpElement > element ( alib::xmlApi < regexp::FormalRegExpElement >::parse ( input ) );
	FormalRegExp regexp ( std::move ( alphabet ), std::move ( element ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, FormalRegExp::getXmlTagName() );

	return regexp;
}

void FormalRegExp::compose ( std::deque < sax::Token > & out ) const {
	out.emplace_back ( FormalRegExp::getXmlTagName(), sax::Token::TokenType::START_ELEMENT );
	RegExpToXMLComposer::composeAlphabet ( out, getAlphabet ( ) );
	alib::xmlApi < FormalRegExpElement >::compose ( out, this->getRegExp ( ) );
	out.emplace_back ( FormalRegExp::getXmlTagName(), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace regexp */

namespace alib {

auto formalRegExpParserRegister = xmlApi < regexp::RegExp >::ParserRegister < regexp::FormalRegExp > ( );
auto formalRegExpParserRegister2 = xmlApi < alib::Object >::ParserRegister < regexp::FormalRegExp > ( );

auto FormalRegExpFromUnboundedRegExp = castApi::CastRegister < regexp::FormalRegExp, regexp::UnboundedRegExp > ( );
auto FormalRegExpCastBinder = castApi::CastPoolStringBinder < regexp::FormalRegExp > ( regexp::FormalRegExp::getXmlTagName() );

} /* namespace alib */
