/*
 * FormalRegExpConcatenation.cpp
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#include "FormalRegExpConcatenation.h"
#include <exception/CommonException.h>
#include "../unbounded/UnboundedRegExpConcatenation.h"

namespace regexp {

FormalRegExpConcatenation::FormalRegExpConcatenation ( FormalRegExpElement && leftElement, FormalRegExpElement && rightElement ) : BinaryNode < std::smart_ptr < FormalRegExpElement >, std::smart_ptr < const FormalRegExpElement >, FormalRegExpConcatenation > ( std::smart_ptr < FormalRegExpElement > ( std::move ( leftElement ).plunder ( ) ), std::smart_ptr < FormalRegExpElement > ( std::move ( rightElement ).plunder ( ) ) ) {
}

FormalRegExpConcatenation::FormalRegExpConcatenation ( const FormalRegExpElement & left, const FormalRegExpElement & right ) : FormalRegExpConcatenation ( std::move_copy ( left ), std::move_copy ( right ) ) {
}

const FormalRegExpElement & FormalRegExpConcatenation::getLeftElement ( ) const {
	return * getLeft ( );
}

const FormalRegExpElement & FormalRegExpConcatenation::getRightElement ( ) const {
	return * getRight ( );
}

FormalRegExpElement & FormalRegExpConcatenation::getLeftElement ( ) {
	return * getLeft ( );
}

FormalRegExpElement & FormalRegExpConcatenation::getRightElement ( ) {
	return * getRight ( );
}

void FormalRegExpConcatenation::setLeftElement ( FormalRegExpElement && element ) {
	setLeft( std::smart_ptr < FormalRegExpElement > ( std::move ( element ).plunder ( ) ) );
}

void FormalRegExpConcatenation::setLeftElement ( const FormalRegExpElement & element ) {
	setLeftElement ( std::move_copy ( element ) );
}

void FormalRegExpConcatenation::setRightElement ( FormalRegExpElement && element ) {
	setRight ( std::smart_ptr < FormalRegExpElement > ( std::move ( element ).plunder ( ) ) );
}

void FormalRegExpConcatenation::setRightElement ( const FormalRegExpElement & element ) {
	setRightElement ( std::move_copy ( element ) );
}

FormalRegExpElement * FormalRegExpConcatenation::clone ( ) const {
	return new FormalRegExpConcatenation ( * this );
}

std::smart_ptr < UnboundedRegExpElement > FormalRegExpConcatenation::asUnbounded ( ) const {
	UnboundedRegExpConcatenation * res = new UnboundedRegExpConcatenation ( );

	res->appendElement ( std::move ( * getLeftElement ( ).asUnbounded ( ) ) );
	res->appendElement ( std::move ( * getRightElement ( ).asUnbounded ( ) ) );

	return std::smart_ptr < UnboundedRegExpElement > ( res );
}

FormalRegExpElement * FormalRegExpConcatenation::plunder ( ) && {
	return new FormalRegExpConcatenation ( std::move ( * this ) );
}

int FormalRegExpConcatenation::compare ( const FormalRegExpConcatenation & other ) const {
	int res = getLeftElement ( ).compare ( other.getLeftElement ( ) );

	if ( res == 0 ) res = getRightElement ( ).compare ( other.getRightElement ( ) );

	return res;
}

void FormalRegExpConcatenation::operator >>( std::ostream & out ) const {
	out << "(FormalRegExpConcatenation";
	out << " " << getLeft ( );
	out << " " << getRight ( );
	out << ")";
}

bool FormalRegExpConcatenation::testSymbol ( const alphabet::Symbol & symbol ) const {
	return getLeft ( )->testSymbol ( symbol ) && getRight ( )->testSymbol ( symbol );
}

void FormalRegExpConcatenation::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	getLeftElement ( ).computeMinimalAlphabet ( alphabet );
	getRightElement ( ).computeMinimalAlphabet ( alphabet );
}

bool FormalRegExpConcatenation::checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const {
	return getLeftElement ( ).checkAlphabet ( alphabet ) && getRightElement ( ).checkAlphabet ( alphabet );
}

FormalRegExpConcatenation::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
