/*
 * FormalRegExpSymbol.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "FormalRegExpSymbol.h"
#include "../unbounded/UnboundedRegExpSymbol.h"
#include <sstream>

namespace regexp {

FormalRegExpSymbol::FormalRegExpSymbol ( int number ) : symbol ( alphabet::symbolFrom ( number ) ) {
}

FormalRegExpSymbol::FormalRegExpSymbol ( char character ) : symbol ( alphabet::symbolFrom ( character ) ) {
}

FormalRegExpSymbol::FormalRegExpSymbol ( const std::string & label ) : symbol ( alphabet::symbolFrom ( label ) ) {
}

FormalRegExpSymbol::FormalRegExpSymbol ( alphabet::Symbol symbol ) : symbol ( std::move ( symbol ) ) {
}

FormalRegExpElement * FormalRegExpSymbol::clone ( ) const {
	return new FormalRegExpSymbol ( * this );
}

FormalRegExpElement * FormalRegExpSymbol::plunder ( ) && {
	return new FormalRegExpSymbol ( std::move ( * this ) );
}

std::smart_ptr < UnboundedRegExpElement > FormalRegExpSymbol::asUnbounded ( ) const {
	return std::smart_ptr < UnboundedRegExpElement > ( new UnboundedRegExpSymbol ( this->symbol ) );
}

int FormalRegExpSymbol::compare ( const FormalRegExpSymbol & other ) const {
	return symbol.getData ( ).compare ( other.symbol.getData ( ) );
}

void FormalRegExpSymbol::operator >>( std::ostream & out ) const {
	out << "(FormalRegExpSymbol " << symbol << ")";
}

bool FormalRegExpSymbol::testSymbol ( const alphabet::Symbol & symbol ) const {
	return symbol == this->symbol;
}

void FormalRegExpSymbol::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	alphabet.insert ( symbol );
}

bool FormalRegExpSymbol::checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const {
	return alphabet.count ( symbol );
}

const alphabet::Symbol & FormalRegExpSymbol::getSymbol ( ) const {
	return symbol;
}

FormalRegExpSymbol::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
