/*
 * FormalRegExpSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef FORMAL_REG_EXP_SYMBOL_H_
#define FORMAL_REG_EXP_SYMBOL_H_

#include "FormalRegExpElement.h"
#include "FormalRegExp.h"

namespace regexp {

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
class FormalRegExpSymbol : public FormalRegExpElement, public std::NullaryNode < std::smart_ptr < FormalRegExpElement >, std::smart_ptr < const FormalRegExpElement >, FormalRegExpSymbol > {
	alphabet::Symbol symbol;

public:
	void Accept ( void * userData, const FormalRegExpElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRegExpSymbol ( int number ); // TODO remove these
	explicit FormalRegExpSymbol ( char character );
	explicit FormalRegExpSymbol ( const std::string & label );

	explicit FormalRegExpSymbol ( alphabet::Symbol symbol );

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement * clone ( ) const;

	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement * plunder ( ) &&;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual std::smart_ptr < UnboundedRegExpElement > asUnbounded ( ) const;

	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @copydoc FormalRegExpElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const;

	virtual int compare ( const FormalRegExpElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRegExpSymbol & ) const;

	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	/**
	 * Returns the string representation of RegExp Symbol.
	 */
	const alphabet::Symbol & getSymbol ( ) const;

	virtual explicit operator std::string ( ) const;
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_SYMBOL_H_ */
