/*
 * UnboundedRegExpAlternation.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef UNBOUNDED_REG_EXP_ALTERNATION_H_
#define UNBOUNDED_REG_EXP_ALTERNATION_H_

#include <vector>
#include "UnboundedRegExpElement.h"

namespace regexp {

namespace simplify {

class RegExpOptimize;

} /* namespace simplify */

/**
 * Represents alternation operator in the regular expression. Contains list of UnboundedRegExpElement
 * as operands of the operator.
 */
class UnboundedRegExpAlternation : public UnboundedRegExpElement, public std::VararyNode < std::smart_ptr < UnboundedRegExpElement >, std::smart_ptr < const UnboundedRegExpElement >, UnboundedRegExpAlternation > {
public:
	void Accept ( void * userData, const UnboundedRegExpElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit UnboundedRegExpAlternation ( );

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement * clone ( ) const;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement * plunder ( ) &&;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual std::smart_ptr < FormalRegExpElement > asFormal ( ) const;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @copydoc UnboundedRegExpElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @return elements
	 */
	const std::vector < std::smart_ptr < const UnboundedRegExpElement > > & getElements ( ) const;

	/**
	 * @return elements
	 */
	const std::vector < std::smart_ptr < UnboundedRegExpElement > > & getElements ( );

	/**
	 * @param element to append
	 */
	void appendElement ( UnboundedRegExpElement && element );
	void appendElement ( const UnboundedRegExpElement & element );

	virtual int compare ( const UnboundedRegExpElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UnboundedRegExpAlternation & ) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	friend class simplify::RegExpOptimize;

	virtual explicit operator std::string ( ) const;
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_ALTERNATION_H_ */
