/*
 * UnboundedRegExpIteration.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef UNBOUNDED_REG_EXP_ITERATION_H_
#define UNBOUNDED_REG_EXP_ITERATION_H_

#include "UnboundedRegExpElement.h"
#include "UnboundedRegExpEmpty.h"

namespace regexp {

namespace simplify {

class RegExpOptimize;

} /* namespace simplify */

/**
 * Represents iteration operator in the regular expression. Contains one UnboundedRegExpElement
 * as operand.
 */
class UnboundedRegExpIteration : public UnboundedRegExpElement, public std::UnaryNode < std::smart_ptr < UnboundedRegExpElement >, std::smart_ptr < const UnboundedRegExpElement >, UnboundedRegExpIteration > {
public:
	void Accept ( void * userData, const UnboundedRegExpElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit UnboundedRegExpIteration ( UnboundedRegExpElement && );
	explicit UnboundedRegExpIteration ( const UnboundedRegExpElement & );

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement * clone ( ) const;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement * plunder ( ) &&;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual std::smart_ptr < FormalRegExpElement > asFormal ( ) const;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @copydoc UnboundedRegExpElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @return element
	 */
	const UnboundedRegExpElement & getElement ( ) const;

	/**
	 * @return element
	 */
	UnboundedRegExpElement & getElement ( );

	/**
	 * @param element to iterate
	 */
	void setElement ( UnboundedRegExpElement && element );
	void setElement ( const UnboundedRegExpElement & element );

	virtual int compare ( const UnboundedRegExpElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UnboundedRegExpIteration & ) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	friend class simplify::RegExpOptimize;

	virtual explicit operator std::string ( ) const;
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_ITERATION_H_ */
