/*
 * UnboundedRegExpSymbol.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpSymbol.h"
#include "../formal/FormalRegExpSymbol.h"
#include <sstream>

namespace regexp {

UnboundedRegExpSymbol::UnboundedRegExpSymbol ( int number ) : symbol ( alphabet::symbolFrom ( number ) ) {
}

UnboundedRegExpSymbol::UnboundedRegExpSymbol ( char character ) : symbol ( alphabet::symbolFrom ( character ) ) {
}

UnboundedRegExpSymbol::UnboundedRegExpSymbol ( const std::string & label ) : symbol ( alphabet::symbolFrom ( label ) ) {
}

UnboundedRegExpSymbol::UnboundedRegExpSymbol ( alphabet::Symbol symbol ) : symbol ( std::move ( symbol ) ) {
}

UnboundedRegExpElement * UnboundedRegExpSymbol::clone ( ) const {
	return new UnboundedRegExpSymbol ( * this );
}

UnboundedRegExpElement * UnboundedRegExpSymbol::plunder ( ) && {
	return new UnboundedRegExpSymbol ( std::move ( * this ) );
}

std::smart_ptr < FormalRegExpElement > UnboundedRegExpSymbol::asFormal ( ) const {
	return std::smart_ptr < FormalRegExpElement > ( new FormalRegExpSymbol ( this->symbol ) );
}

int UnboundedRegExpSymbol::compare ( const UnboundedRegExpSymbol & other ) const {
	return symbol.getData ( ).compare ( other.symbol.getData ( ) );
}

void UnboundedRegExpSymbol::operator >>( std::ostream & out ) const {
	out << "(UnboundedRegExpSymbol " << symbol << ")";
}

bool UnboundedRegExpSymbol::testSymbol ( const alphabet::Symbol & symbol ) const {
	return symbol == this->symbol;
}

bool UnboundedRegExpSymbol::checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const {
	return alphabet.count ( symbol );
}

void UnboundedRegExpSymbol::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	alphabet.insert ( this->symbol );
}

const alphabet::Symbol & UnboundedRegExpSymbol::getSymbol ( ) const {
	return this->symbol;
}

UnboundedRegExpSymbol::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
