#include "FormalRTE.h"
#include "FormalRTEEmpty.h"
#include "FormalRTESymbol.h"

#include <iostream>
#include <algorithm>
#include <sstream>

// #include "../unbounded/UnboundedRTE.h"

#include <sax/FromXMLParserHelper.h>
#include "../common/RTEFromXMLParser.h"
#include "../common/RTEToXMLComposer.h"
#include "../RTE.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include <cast/CastApi.hpp>

namespace rte {

FormalRTE::FormalRTE ( std::set < alphabet::RankedSymbol > alphabetF, std::set < alphabet::RankedSymbol > alphabetK, FormalRTEElement && rte ) : std::Components < FormalRTE, alphabet::RankedSymbol, std::tuple < GeneralAlphabet, ConstantAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( alphabetF ), std::move ( alphabetK ) ), std::tuple < > ( ) ), rte ( NULL ) {
	setRTE ( std::move ( rte ) );
}

FormalRTE::FormalRTE ( std::pair < std::set < alphabet::RankedSymbol >, std::set < alphabet::RankedSymbol > > alphabets, FormalRTEElement && rte ) : FormalRTE ( std::move ( alphabets.first ), std::move ( alphabets.second ), std::move ( rte ) ) {
}

FormalRTE::FormalRTE ( std::pair < std::set < alphabet::RankedSymbol >, std::set < alphabet::RankedSymbol > > alphabets, const FormalRTEElement & rte ) : FormalRTE ( std::move ( alphabets.first ), std::move ( alphabets.second ), std::move_copy ( rte ) ) {
}

FormalRTE::FormalRTE ( ) : FormalRTE ( std::set < alphabet::RankedSymbol > ( ), std::set < alphabet::RankedSymbol > ( ), FormalRTEEmpty ( ) ) {
}

/*
 * FormalRTE::FormalRTE ( const UnboundedRTE & other ) {
 *  alphabet = other.getAlphabet ( );
 *  this->rte = NULL;
 *  FormalRTEElement * element = other.getRTE ( ).cloneAsFormal ( );
 *  setRTE ( std::move ( * element ) );
 *  delete element;
 * }
 */

FormalRTE::FormalRTE ( std::set < alphabet::RankedSymbol > alphabetF, std::set < alphabet::RankedSymbol > alphabetK, const FormalRTEElement & rte ) : FormalRTE ( std::move ( alphabetF ), std::move ( alphabetK ), std::move_copy ( rte ) ) {
}

FormalRTE::FormalRTE ( const FormalRTEElement & rte ) : FormalRTE ( rte.computeMinimalAlphabets ( ), std::move_copy ( rte ) ) {
}

FormalRTE::FormalRTE ( FormalRTEElement && rte ) : FormalRTE ( rte.computeMinimalAlphabets ( ), rte ) {
}

FormalRTE::FormalRTE ( const FormalRTE & other ) : std::Components < FormalRTE, alphabet::RankedSymbol, std::tuple < GeneralAlphabet, ConstantAlphabet >, std::tuple < > > ( std::make_tuple ( other.getAlphabet ( ), other.getConstantAlphabet ( ) ), std::tuple < > ( ) ), rte ( other.rte ) {
	this->rte->attachRTE ( this );
}

FormalRTE::FormalRTE ( FormalRTE && other ) noexcept : std::Components < FormalRTE, alphabet::RankedSymbol, std::tuple < GeneralAlphabet, ConstantAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( other.accessComponent < GeneralAlphabet > ( ).get ( ) ), std::move ( other.accessComponent < ConstantAlphabet > ( ).get ( ) ) ), std::tuple < > ( ) ), rte ( std::move ( other.rte ) ) {
	this->rte->attachRTE ( this );
}

RTEBase * FormalRTE::clone ( ) const {
	return new FormalRTE ( * this );
}

RTEBase * FormalRTE::plunder ( ) && {
	return new FormalRTE ( std::move ( * this ) );
}

FormalRTE & FormalRTE::operator =( const FormalRTE & other ) {
	if ( this == & other )
		return * this;

	* this = FormalRTE ( other );

	return * this;
}

FormalRTE & FormalRTE::operator =( FormalRTE && other ) noexcept {
	std::swap ( this->rte, other.rte );
	std::swap ( accessComponent < GeneralAlphabet > ( ).get ( ), other.accessComponent < GeneralAlphabet > ( ).get ( ) );
	std::swap ( accessComponent < ConstantAlphabet > ( ).get ( ), other.accessComponent < ConstantAlphabet > ( ).get ( ) );

	return * this;
}

FormalRTE::~FormalRTE ( ) noexcept {
}

const FormalRTEElement & FormalRTE::getRTE ( ) const {
	return * rte;
}

FormalRTEElement & FormalRTE::getRTE ( ) {
	return * rte;
}

void FormalRTE::setRTE ( const FormalRTEElement & rte ) {
	setRTE ( std::move_copy ( rte ) );
}

void FormalRTE::setRTE ( FormalRTEElement && param ) {
	this->rte = std::smart_ptr < FormalRTEElement > ( std::move ( param ).plunder ( ) );

	if ( !this->rte->attachRTE ( this ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );
}

void FormalRTE::operator >>( std::ostream & out ) const {
	out << "(FormalRTE " << * ( this->rte ) << ")";
}

int FormalRTE::compare ( const FormalRTE & other ) const {
	auto first = std::tie ( rte, getAlphabet ( ), getConstantAlphabet ( ) );
	auto second = std::tie ( rte, other.getAlphabet ( ), getConstantAlphabet ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

FormalRTE::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

FormalRTE FormalRTE::parse ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, FormalRTE::getXmlTagName() );

	std::pair < std::set < alphabet::RankedSymbol >, std::set < alphabet::RankedSymbol > > alphabets = RTEFromXMLParser::parseAlphabet ( input );
	std::rvalue_ref < FormalRTEElement > element = alib::xmlApi < rte::FormalRTEElement >::parse ( input );
	FormalRTE rte ( std::move ( alphabets.first ), std::move ( alphabets.second ), std::move ( element ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, FormalRTE::getXmlTagName() );
	return rte;
}

void FormalRTE::compose ( std::deque < sax::Token > & out ) const {
	out.emplace_back ( FormalRTE::getXmlTagName(), sax::Token::TokenType::START_ELEMENT );
	RTEToXMLComposer::composeAlphabet ( out, this->getAlphabet ( ), this->getConstantAlphabet ( ) );
	alib::xmlApi < FormalRTEElement >::compose ( out, this->getRTE ( ) );
	out.emplace_back ( FormalRTE::getXmlTagName(), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace rte */

namespace alib {

auto formalRTEParserRegister  = xmlApi < rte::RTE >::ParserRegister < rte::FormalRTE > ( );
auto formalRTEParserRegister2 = xmlApi < alib::Object >::ParserRegister < rte::FormalRTE > ( );

 // auto FormalRTEFromUnboundedRTE = castApi::CastRegister < rte::FormalRTE, rte::UnboundedRTE > ( );
auto FormalRTECastBinder = castApi::CastPoolStringBinder < rte::FormalRTE > ( rte::FormalRTE::getXmlTagName() );

} /* namespace alib */
