#include "FormalRTEAlternation.h"
#include <exception/CommonException.h>

// #include "../unbounded/UnboundedRTEAlternation.h"
#include <sstream>

namespace rte {

FormalRTEAlternation::FormalRTEAlternation ( FormalRTEElement && leftElement, FormalRTEElement && rightElement ) : left ( nullptr ), right ( nullptr ) {
	setLeftElement ( std::move ( leftElement ) );
	setRightElement ( std::move ( rightElement ) );
}

FormalRTEAlternation::FormalRTEAlternation ( const FormalRTEElement & left, const FormalRTEElement & right ) : FormalRTEAlternation ( std::move_copy ( left ), std::move_copy ( right ) ) {
}

FormalRTEAlternation::FormalRTEAlternation ( const FormalRTEAlternation & other ) : left ( other.left ), right ( other.right ) {
}

FormalRTEAlternation::FormalRTEAlternation ( FormalRTEAlternation && other ) noexcept : left ( std::move ( other.left ) ), right ( std::move ( other.right ) ) {
	left->attachRTE ( NULL );
	right->attachRTE ( NULL );
}

FormalRTEAlternation & FormalRTEAlternation::operator =( const FormalRTEAlternation & other ) {
	if ( this == & other )
		return * this;

	* this = FormalRTEAlternation ( other );

	return * this;
}

FormalRTEAlternation & FormalRTEAlternation::operator =( FormalRTEAlternation && other ) noexcept {
	std::swap ( this->left, other.left );
	std::swap ( this->right, other.right );

	this->left->attachRTE ( this->parentRTE );
	this->right->attachRTE ( this->parentRTE );

	return * this;
}

FormalRTEAlternation::~FormalRTEAlternation ( ) noexcept {
}

const FormalRTEElement & FormalRTEAlternation::getLeftElement ( ) const {
	return * left;
}

const FormalRTEElement & FormalRTEAlternation::getRightElement ( ) const {
	return * right;
}

FormalRTEElement & FormalRTEAlternation::getLeftElement ( ) {
	return * left;
}

FormalRTEElement & FormalRTEAlternation::getRightElement ( ) {
	return * right;
}

void FormalRTEAlternation::setLeftElement ( const FormalRTEElement & element ) {
	setLeftElement ( std::move_copy ( element ) );
}

void FormalRTEAlternation::setLeftElement ( FormalRTEElement && element ) {
	this->left = std::smart_ptr < FormalRTEElement > ( std::move ( element ).plunder ( ) );

	if ( this->parentRTE && !left->attachRTE ( this->parentRTE ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );
}

void FormalRTEAlternation::setRightElement ( const FormalRTEElement & element ) {
	setRightElement ( std::move_copy ( element ) );
}

void FormalRTEAlternation::setRightElement ( FormalRTEElement && element ) {
	this->right = std::smart_ptr < FormalRTEElement > ( std::move ( element ).plunder ( ) );

	if ( this->parentRTE && !right->attachRTE ( this->parentRTE ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );
}

FormalRTEAlternation * FormalRTEAlternation::clone ( ) const {
	return new FormalRTEAlternation ( * this );
}

FormalRTEAlternation * FormalRTEAlternation::plunder ( ) && {
	return new FormalRTEAlternation ( std::move ( * this ) );
}

/*
 * UnboundedRTEElement* FormalRTEAlternation::cloneAsUnbounded() const {
 *  UnboundedRTEAlternation* res = new UnboundedRTEAlternation();
 *
 *  UnboundedRTEElement* leftTmp = left->cloneAsUnbounded();
 *  res->appendElement(std::move(*leftTmp));
 *  delete leftTmp;
 *
 *  UnboundedRTEElement* rightTmp = right->cloneAsUnbounded();
 *  res->appendElement(std::move(*rightTmp));
 *  delete rightTmp;
 *
 *  return res;
 * }
 */
int FormalRTEAlternation::compare ( const FormalRTEAlternation & other ) const {
	int res = left->compare ( * other.left );

	if ( res == 0 ) res = right->compare ( * other.right );

	return res;
}

void FormalRTEAlternation::operator >>( std::ostream & out ) const {
	out << "(FormalRTEAlternation";
	out << " " << * left;
	out << " " << * right;
	out << ")";
}

bool FormalRTEAlternation::testSymbol ( const alphabet::RankedSymbol & symbol ) const {
	if ( left->testSymbol ( symbol ) ) return true;

	if ( right->testSymbol ( symbol ) ) return true;

	return false;
}

bool FormalRTEAlternation::attachRTE ( const FormalRTE * regexp ) {
	if ( this->parentRTE == regexp ) return true;

	this->parentRTE = regexp;

	if ( !left->attachRTE ( regexp ) ) return false;

	if ( !right->attachRTE ( regexp ) ) return false;

	return true;
}

void FormalRTEAlternation::computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const {
	left->computeMinimalAlphabet ( alphabetF, alphabetK );
	right->computeMinimalAlphabet ( alphabetF, alphabetK );
}

FormalRTEAlternation::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
