#ifndef FORMAL_RTE_ELEMENT_H_
#define FORMAL_RTE_ELEMENT_H_

#include <XmlApi.hpp>
#include "../../alphabet/Symbol.h"
#include "../common/RTEAlphabet.h"
#include <set>

namespace rte {

class FormalRTE;

class FormalRTEAlternation;
class FormalRTEIteration;
class FormalRTESubstitution;
class FormalRTESymbol;
class FormalRTEEmpty;

 // class UnboundedRTEElement;
class FormalRTEElement;

class FormalRTEElementVisitor {
public:
	virtual void Visit(void*, const FormalRTEAlternation& alternation) const = 0;
	virtual void Visit(void*, const FormalRTEIteration& iteration) const = 0;
	virtual void Visit(void*, const FormalRTESubstitution& concatenation) const = 0;
	virtual void Visit(void*, const FormalRTESymbol& symbol) const = 0;
	virtual void Visit(void*, const FormalRTEEmpty& empty) const = 0;
};

class FormalRTEElement : public alib::CommonBase < FormalRTEElement > {
protected:
	/*
	 * Parent regexp contanining this instance of RTEElement
	 */
	const FormalRTE * parentRTE;

	/**
	 * Attaches the regexp to this instance and all its childs
	 * @param regexp parent regexp to attach as parent
	 * @return true if symbols used in regexp element are in the regexp's alphabet
	 */
	virtual bool attachRTE ( const FormalRTE * regexp ) = 0;

public:
	virtual void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const = 0;

	explicit FormalRTEElement ( );

	/**
	 * Traverses the regexp tree looking if particular Symbol is used in the regexp.
	 *
	 * @param symbol to test if used in regexp element
	 * @return true if symbol is used by the element and its successor
	 */
	virtual bool testSymbol ( const alphabet::RankedSymbol & symbol ) const = 0;

	/**
	 * Traverses the regexp tree computing minimal alphabet needed by regexp
	 *
	 * @param alphabet All alphabet symbols encountered are added into this set
	 * @return true if symbol is used by the element and its successor
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const = 0;

	/**
	 * @copydoc RankedNode::computeMinimalAlphabet()
	 */
	std::pair < std::set < alphabet::RankedSymbol >, std::set < alphabet::RankedSymbol > > computeMinimalAlphabets ( ) const;

	/**
	 * Creates copy of the element.
	 * @return copy of the element
	 */

	// virtual UnboundedRTEElement * cloneAsUnbounded ( ) const = 0;

	friend class FormalRTE;

	friend class FormalRTEAlternation;
	friend class FormalRTEIteration;
	friend class FormalRTESubstitution;
	friend class FromalRTESymbol;
	friend class FormalRTEEmpty;
};

} /* namespace rte */

namespace alib {

template < >
struct xmlApi < rte::FormalRTEElement > {
	static std::rvalue_ref < rte::FormalRTEElement > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, const rte::FormalRTEElement & data );
};

} /* namespace alib */

#endif /* FORMAL_RTE_ELEMENT_H_ */
