#ifndef FORMAL_RTE_EMPTY_H_
#define FORMAL_RTE_EMPTY_H_

#include "FormalRTEElement.h"
#include <sstream>

namespace rte {

/**
 * Represents empty regular expression in the regular expression.
 */
class FormalRTEEmpty : public FormalRTEElement {
protected:
	/**
	 * @copydoc FormalRTEElement::clone() const
	 */

	// virtual UnboundedRTEElement * cloneAsUnbounded ( ) const;

	/**
	 * @copydoc FormalRTEElement::attachRTE()
	 */
	virtual bool attachRTE ( const FormalRTE * regexp );

public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTEEmpty ( );
	FormalRTEEmpty ( const FormalRTEEmpty & other );
	FormalRTEEmpty ( FormalRTEEmpty && other ) noexcept;
	FormalRTEEmpty & operator =( const FormalRTEEmpty & other );
	FormalRTEEmpty & operator =( FormalRTEEmpty && other ) noexcept;

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTEEmpty * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTEEmpty * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::RankedSymbol & symbol ) const;

	/**
	 * @copydoc RTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const;

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTEEmpty & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;
};

} /* namespace rte */

#endif /* FORMAL_RTE_EMPTY_H_ */
