#include "FormalRTESymbol.h"

// #include "../unbounded/UnboundedRTESymbol.h"
#include <exception/CommonException.h>
#include "../../primitive/Unsigned.h"
#include <sstream>

namespace rte {

FormalRTESymbol::FormalRTESymbol ( const FormalRTESymbol & other ) : symbol ( other.symbol ), elements ( std::move ( other.elements ) ) {
}

FormalRTESymbol::FormalRTESymbol ( FormalRTESymbol && other ) noexcept : symbol ( std::move ( other.symbol ) ), elements ( std::move ( other.elements ) ) {
	this->attachRTE ( NULL );
}

FormalRTESymbol & FormalRTESymbol::operator =( const FormalRTESymbol & other ) {
	if ( this == & other )
		return * this;

	* this = FormalRTESymbol ( other );

	return * this;
}

FormalRTESymbol & FormalRTESymbol::operator =( FormalRTESymbol && other ) noexcept {
	std::swap ( this->symbol, other.symbol );
	std::swap ( this->elements, other.elements );
	std::swap ( this->parentRTE, other.parentRTE );

	this->attachRTE ( other.parentRTE );

	return * this;
}

FormalRTESymbol::FormalRTESymbol ( alphabet::RankedSymbol symbol ) : symbol ( std::move ( symbol ) ) {
}

FormalRTESymbol * FormalRTESymbol::clone ( ) const {
	return new FormalRTESymbol ( * this );
}

FormalRTESymbol * FormalRTESymbol::plunder ( ) && {
	return new FormalRTESymbol ( std::move ( * this ) );
}

const std::vector < std::smart_ptr < const FormalRTESymbol > > & FormalRTESymbol::getElements ( ) const {
	return reinterpret_cast < const std::vector < std::smart_ptr < const FormalRTESymbol > > & > ( elements );
}

std::vector < std::smart_ptr < FormalRTESymbol > > & FormalRTESymbol::getElements ( ) {
	return elements;
}

void FormalRTESymbol::appendElement ( FormalRTESymbol element ) {
	if ( primitive::Unsigned ( this->elements.size ( ) ) >= this->symbol.getRank ( ) )
		throw exception::CommonException ( "Ranked node cannot have more children then its rank" );

	std::smart_ptr < FormalRTESymbol > elem ( std::move ( element ).plunder ( ) );

	if ( this->parentRTE && !elem->attachRTE ( this->parentRTE ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );

	this->elements.push_back ( std::move ( elem ) );
}

/*
 * UnboundedRTEElement* FormalRTESymbol::cloneAsUnbounded() const {
 *  return new UnboundedRTESymbol(this->symbol);
 * }
 */

int FormalRTESymbol::compare ( const FormalRTESymbol & other ) const {
	auto first = std::tie ( symbol, getElements ( ) );
	auto second = std::tie ( other.symbol, other.getElements ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

void FormalRTESymbol::operator >>( std::ostream & out ) const {
	out << "(FormalRTESymbol " << " symbol = " << this->symbol << " children = " << this->elements << "})";
}

bool FormalRTESymbol::testSymbol ( const alphabet::RankedSymbol & symbol ) const {
	return symbol == this->symbol;
}

bool FormalRTESymbol::attachRTE ( const FormalRTE * regexp ) {
	if ( this->parentRTE == regexp ) return true;

	this->parentRTE = regexp;

	if ( regexp == NULL ) return true;

	for ( const std::smart_ptr < FormalRTESymbol > & element : this->elements )
		if ( !element->attachRTE ( regexp ) ) return false;

	return this->parentRTE->getAlphabet ( ).find ( this->symbol ) != this->parentRTE->getAlphabet ( ).end ( ) || this->parentRTE->getConstantAlphabet ( ).find ( this->symbol ) != this->parentRTE->getConstantAlphabet ( ).end ( );
}

void FormalRTESymbol::computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & /* alphabetK */ ) const {
	alphabetF.insert ( this->symbol );
}

const alphabet::RankedSymbol & FormalRTESymbol::getSymbol ( ) const {
	return this->symbol;
}

FormalRTESymbol::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
