#ifndef FORMAL_RTE_SYMBOL_H_
#define FORMAL_RTE_SYMBOL_H_

#include "../../label/Label.h"
#include "../../alphabet/LabeledSymbol.h"
#include "FormalRTEElement.h"
#include "FormalRTE.h"

namespace rte {

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
class FormalRTESymbol : public FormalRTEElement {
protected:
	alphabet::RankedSymbol symbol;
	std::vector < std::smart_ptr < FormalRTESymbol > > elements; // can be empty (leaf)

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */

	// virtual UnboundedRTEElement * cloneAsUnbounded ( ) const;

	/**
	 * @copydoc FormalRTEElement::attachRTE()
	 */
	virtual bool attachRTE ( const FormalRTE * regexp );

public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTESymbol ( alphabet::RankedSymbol symbol );

	FormalRTESymbol ( const FormalRTESymbol & other );
	FormalRTESymbol ( FormalRTESymbol && other ) noexcept;
	FormalRTESymbol & operator =( const FormalRTESymbol & other );
	FormalRTESymbol & operator =( FormalRTESymbol && other ) noexcept;

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTESymbol * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTESymbol * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::RankedSymbol & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const;

	const std::vector < std::smart_ptr < const FormalRTESymbol > > & getElements ( ) const;
	std::vector < std::smart_ptr < FormalRTESymbol > > & getElements ( );

	void appendElement ( FormalRTESymbol element );

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTESymbol & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	/**
	 * Returns the string representation of RTE Symbol.
	 */
	const alphabet::RankedSymbol & getSymbol ( ) const;

	virtual explicit operator std::string ( ) const;

	friend class FormalRTEIteration;
	friend class FormalRTESubstitution;
};

} /* namespace rte */

#endif /* FORMAL_RTE_SYMBOL_H_ */
