/*
 * TreeFromXMLParser.cpp
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#include <sax/FromXMLParserHelper.h>
#include "TreeFromXMLParser.h"
#include <sax/ParserException.h>
#include <XmlApi.hpp>

namespace tree {

alphabet::Symbol TreeFromXMLParser::parseBar ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "bar" );
	alphabet::Symbol bar ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "bar" );

	return bar;
}

std::set < alphabet::RankedSymbol > TreeFromXMLParser::parseBars ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::RankedSymbol > bars;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "bars" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		bars.insert ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "bars" );

	return bars;
}

alphabet::Symbol TreeFromXMLParser::parseVariablesBar ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "variablesBar" );
	alphabet::Symbol variablesBar ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "variablesBar" );

	return variablesBar;
}

alphabet::RankedSymbol TreeFromXMLParser::parseRankedVariablesBar ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "RankedVariablesBar" );
	alphabet::RankedSymbol rankedVariablesBar ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "RankedVariablesBar" );

	return rankedVariablesBar;
}

alphabet::Symbol TreeFromXMLParser::parseSubtreeWildcardSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "subtreeWildcard" );
	alphabet::Symbol subtreeWildcard ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "subtreeWildcard" );

	return subtreeWildcard;
}

alphabet::RankedSymbol TreeFromXMLParser::parseSubtreeWildcardRankedSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "subtreeWildcard" );
	alphabet::RankedSymbol subtreeWildcard ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "subtreeWildcard" );

	return subtreeWildcard;
}

std::set < alphabet::RankedSymbol > TreeFromXMLParser::parseRankedAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::RankedSymbol > rankedSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "rankedAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		rankedSymbols.insert ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "rankedAlphabet" );
	return rankedSymbols;
}

std::set < alphabet::Symbol > TreeFromXMLParser::parseUnrankedAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > symbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		symbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );
	return symbols;
}

std::vector < alphabet::RankedSymbol > TreeFromXMLParser::parseRankedContent ( std::deque < sax::Token >::iterator & input ) {
	std::vector < alphabet::RankedSymbol > data;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "content" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		data.push_back ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "content" );
	return data;
}

std::vector < alphabet::Symbol > TreeFromXMLParser::parseUnrankedContent ( std::deque < sax::Token >::iterator & input ) {
	std::vector < alphabet::Symbol > data;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "content" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		data.push_back ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "content" );
	return data;
}

RankedNode TreeFromXMLParser::parseRankedNode ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "rankedNode" );
	alphabet::RankedSymbol symbol = alib::xmlApi < alphabet::RankedSymbol >::parse ( input );
	std::vector < std::smart_ptr < RankedNode > > children;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		children.emplace_back ( std::make_smart< RankedNode > ( parseRankedNode ( input ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "rankedNode" );

	return RankedNode ( std::move ( symbol ), std::move ( children ) );
}

UnrankedNode TreeFromXMLParser::parseUnrankedNode ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "unrankedNode" );
	alphabet::Symbol symbol = alib::xmlApi < alphabet::Symbol >::parse ( input );
	std::vector < std::smart_ptr < UnrankedNode > > children;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		children.emplace_back ( std::make_smart< UnrankedNode > ( parseUnrankedNode ( input ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "unrankedNode" );

	return UnrankedNode ( std::move ( symbol ), std::move ( children ) );
}

std::set < alphabet::RankedSymbol > TreeFromXMLParser::parseRankedNonlinearVariables ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::RankedSymbol > rankedSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "rankedNonlinearVariables" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		rankedSymbols.insert ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "rankedNonlinearVariables" );
	return rankedSymbols;
}

std::set < alphabet::Symbol > TreeFromXMLParser::parseUnrankedNonlinearVariables ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > symbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "nonlinearVariables" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		symbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "nonlinearVariables" );
	return symbols;
}

} /* namespace tree */
