/*
 * PrefixRankedBarNonlinearPattern.cpp
 *
 *  Created on: Jul 20, 2016
 *      Author: Jan Travnicek
 */

#include "PrefixRankedBarNonlinearPattern.h"

#include <sstream>
#include <algorithm>
#include <deque>

#include "PrefixRankedBarTree.h"
#include "PrefixRankedBarPattern.h"
#include "RankedTree.h"
#include "RankedPattern.h"
#include "RankedNonlinearPattern.h"

#include <sax/FromXMLParserHelper.h>
#include "../common/TreeFromXMLParser.h"
#include "../common/TreeToXMLComposer.h"
#include "../common/TreeAuxiliary.h"
#include "../Tree.h"
#include "../RankedTreeWrapper.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include <cast/CastApi.hpp>

#include "../../alphabet/BarSymbol.h"
#include "../../alphabet/VariablesBarSymbol.h"
#include "../../alphabet/SubtreeWildcardSymbol.h"

namespace tree {

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( std::set < alphabet::RankedSymbol > bars, alphabet::RankedSymbol variablesBar, alphabet::RankedSymbol subtreeWildcard, std::set < alphabet::RankedSymbol > nonlinearVariables, std::set < alphabet::RankedSymbol > alphabet, std::vector < alphabet::RankedSymbol > data ) : std::Components < PrefixRankedBarNonlinearPattern, alphabet::RankedSymbol, std::tuple < GeneralAlphabet, NonlinearAlphabet, BarSymbols >, std::tuple < SubtreeWildcard, VariablesBarSymbol > > ( std::make_tuple ( std::move ( alphabet ), std::move ( nonlinearVariables ), std::move ( bars ) ), std::make_tuple ( std::move ( subtreeWildcard ), std::move ( variablesBar ) ) ) {
	setContent ( std::move ( data ) );
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( std::set < alphabet::RankedSymbol > bars, alphabet::RankedSymbol variablesBar, alphabet::RankedSymbol subtreeWildcard, std::vector < alphabet::RankedSymbol > data ) : PrefixRankedBarNonlinearPattern ( bars, variablesBar, subtreeWildcard, { }, std::set < alphabet::RankedSymbol > ( data.begin ( ), data.end ( ) ) + bars + std::set < alphabet::RankedSymbol > { variablesBar, subtreeWildcard }, data ) {
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( alphabet::Symbol barBase, alphabet::RankedSymbol variablesBar, const RankedTree & tree ) : PrefixRankedBarNonlinearPattern ( TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + std::set < alphabet::RankedSymbol > { variablesBar }, variablesBar, alphabet::SubtreeWildcardSymbol::RANKED_SUBTREE_WILDCARD_SYMBOL, { }, tree.getAlphabet ( ) + TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + std::set < alphabet::RankedSymbol > { variablesBar, alphabet::SubtreeWildcardSymbol::RANKED_SUBTREE_WILDCARD_SYMBOL }, toPrefixRankedBar ( tree.getRoot ( ), alphabet::SubtreeWildcardSymbol::RANKED_SUBTREE_WILDCARD_SYMBOL, { }, barBase, variablesBar ) ) {
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( alphabet::Symbol barBase, alphabet::RankedSymbol variablesBar, const RankedPattern & tree ) : PrefixRankedBarNonlinearPattern ( TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + std::set < alphabet::RankedSymbol > { variablesBar }, variablesBar, tree.getSubtreeWildcard ( ), { }, tree.getAlphabet ( ) + TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + std::set < alphabet::RankedSymbol > { variablesBar, tree.getSubtreeWildcard ( ) }, toPrefixRankedBar ( tree.getRoot ( ), tree.getSubtreeWildcard ( ), { }, barBase, variablesBar ) ) {
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( alphabet::Symbol barBase, alphabet::RankedSymbol variablesBar, const RankedNonlinearPattern & tree ) : PrefixRankedBarNonlinearPattern ( TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + std::set < alphabet::RankedSymbol > { variablesBar }, variablesBar, tree.getSubtreeWildcard ( ), tree.getNonlinearVariables ( ), tree.getAlphabet ( ) + TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + std::set < alphabet::RankedSymbol > { variablesBar, tree.getSubtreeWildcard ( ) } + tree.getNonlinearVariables ( ), toPrefixRankedBar ( tree.getRoot ( ), tree.getSubtreeWildcard ( ), tree.getNonlinearVariables ( ), barBase, variablesBar ) ) {
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( const PrefixRankedBarTree & tree ) : PrefixRankedBarNonlinearPattern ( tree.getBars ( ) + std::set < alphabet::RankedSymbol > { alphabet::VariablesBarSymbol::RANKED_BAR_SYMBOL }, alphabet::VariablesBarSymbol::RANKED_BAR_SYMBOL, alphabet::SubtreeWildcardSymbol::RANKED_SUBTREE_WILDCARD_SYMBOL, { }, tree.getAlphabet ( ) + std::set < alphabet::RankedSymbol > { alphabet::VariablesBarSymbol::RANKED_BAR_SYMBOL, alphabet::SubtreeWildcardSymbol::RANKED_SUBTREE_WILDCARD_SYMBOL }, tree.getContent ( ) ) {
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( const PrefixRankedBarPattern & tree ) : PrefixRankedBarNonlinearPattern ( tree.getBars ( ), tree.getVariablesBar ( ), tree.getSubtreeWildcard ( ), { }, tree.getAlphabet ( ), tree.getContent ( ) ) {
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( const RankedTree & tree ) : PrefixRankedBarNonlinearPattern ( alphabet::BarSymbol::BAR_SYMBOL, alphabet::VariablesBarSymbol::RANKED_BAR_SYMBOL, tree ) {
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( const RankedPattern & tree ) : PrefixRankedBarNonlinearPattern ( alphabet::BarSymbol::BAR_SYMBOL, alphabet::VariablesBarSymbol::RANKED_BAR_SYMBOL, tree ) {
}

PrefixRankedBarNonlinearPattern::PrefixRankedBarNonlinearPattern ( const RankedNonlinearPattern & tree ) : PrefixRankedBarNonlinearPattern ( alphabet::BarSymbol::BAR_SYMBOL, alphabet::VariablesBarSymbol::RANKED_BAR_SYMBOL, tree ) {
}

std::vector < alphabet::RankedSymbol > PrefixRankedBarNonlinearPattern::toPrefixRankedBar ( const RankedNode & node, const alphabet::RankedSymbol & subtreeWildcard, const std::set < alphabet::RankedSymbol > & nonlinearVariables, const alphabet::Symbol & barBase, const alphabet::RankedSymbol & variablesBar ) {
	std::vector < alphabet::RankedSymbol > res;

	toPrefixRankedBar ( node, subtreeWildcard, nonlinearVariables, barBase, variablesBar, res );
	return res;
}

void PrefixRankedBarNonlinearPattern::toPrefixRankedBar ( const RankedNode & node, const alphabet::RankedSymbol & subtreeWildcard, const std::set < alphabet::RankedSymbol > & nonlinearVariables, const alphabet::Symbol & barBase, const alphabet::RankedSymbol & variablesBar, std::vector < alphabet::RankedSymbol > & linearNotation ) {
	if ( ( node.getSymbol ( ) == subtreeWildcard ) || nonlinearVariables.count ( node.getSymbol ( ) ) ) {
		linearNotation.push_back ( node.getSymbol ( ) );
		linearNotation.push_back ( variablesBar );
	} else {
		linearNotation.push_back ( node.getSymbol ( ) );

		for ( const std::smart_ptr < const RankedNode > & child : node.getChildren ( ) )
			toPrefixRankedBar ( * child, subtreeWildcard, nonlinearVariables, barBase, variablesBar, linearNotation );

		linearNotation.push_back ( alphabet::RankedSymbol ( barBase, node.getSymbol ( ).getRank ( ) ) );
	}
}

RankedTreeBase * PrefixRankedBarNonlinearPattern::clone ( ) const {
	return new PrefixRankedBarNonlinearPattern ( * this );
}

RankedTreeBase * PrefixRankedBarNonlinearPattern::plunder ( ) && {
	return new PrefixRankedBarNonlinearPattern ( std::move ( * this ) );
}

const std::vector < alphabet::RankedSymbol > & PrefixRankedBarNonlinearPattern::getContent ( ) const {
	return this->m_Data;
}

void PrefixRankedBarNonlinearPattern::setContent ( std::vector < alphabet::RankedSymbol > data ) {
	arityChecksum ( data );

	std::set < alphabet::RankedSymbol > minimalAlphabet ( data.begin ( ), data.end ( ) );
	std::set < alphabet::RankedSymbol > unknownSymbols;
	std::set_difference ( minimalAlphabet.begin ( ), minimalAlphabet.end ( ), getAlphabet ( ).begin ( ), getAlphabet ( ).end ( ), std::inserter ( unknownSymbols, unknownSymbols.end ( ) ) );

	if ( unknownSymbols.size ( ) > 0 )
		throw TreeException ( "Input symbols not in the alphabet." );

	this->m_Data = std::move ( data );
}

void PrefixRankedBarNonlinearPattern::arityChecksum ( const std::vector < alphabet::RankedSymbol > & data ) {
	int arityChecksumTerminals = 1;
	int arityChecksumBars = 1;
	int arityChecksumTypes = 0;

	for ( const alphabet::RankedSymbol & symbol : data ) {
		if ( getBars ( ).count ( symbol ) || ( symbol == getVariablesBar ( ) ) ) {
			arityChecksumBars += symbol.getRank ( ).getData ( );
			arityChecksumBars -= 1;
			arityChecksumTypes -= 1;
		} else {
			arityChecksumTerminals += symbol.getRank ( ).getData ( );
			arityChecksumTerminals -= 1;
			arityChecksumTypes += 1;
		}
	}

	if ( ( arityChecksumTerminals != 0 ) || ( arityChecksumBars != 0 ) || ( arityChecksumTypes != 0 ) ) throw TreeException ( "The string does not form a tree" );

	for ( unsigned i = 1; i < data.size ( ); ++i )
		if ( ( data[i - 1] == getSubtreeWildcard ( ) ) && ( data[i] != getVariablesBar ( ) ) )
			throw TreeException ( "Inconsystency of SubtreeWildcard and variablesBar" );

}

bool PrefixRankedBarNonlinearPattern::isEmpty ( ) const {
	return this->m_Data.size ( ) == 0;
}

int PrefixRankedBarNonlinearPattern::compare ( const PrefixRankedBarNonlinearPattern & other ) const {
	auto first = std::tie ( m_Data, getAlphabet ( ), getSubtreeWildcard ( ), getNonlinearVariables ( ), getBars ( ), getVariablesBar ( ) );
	auto second = std::tie ( other.m_Data, other.getAlphabet ( ), other.getSubtreeWildcard ( ), other.getNonlinearVariables ( ), other.getBars ( ), other.getVariablesBar ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

void PrefixRankedBarNonlinearPattern::operator >>( std::ostream & out ) const {
	out << "(PrefixRankedBarNonlinearPattern ";

	for ( const alphabet::RankedSymbol & symbol : this->m_Data )
		out << symbol;

	out << ")";
}

PrefixRankedBarNonlinearPattern::operator std::string ( ) const {
	std::stringstream ss;
	ss << "\"";

	for ( const alphabet::RankedSymbol & symbol : this->m_Data )
		ss << ( std::string ) symbol;

	ss << "\"";
	return std::move ( ss ).str ( );
}

PrefixRankedBarNonlinearPattern PrefixRankedBarNonlinearPattern::parse ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, PrefixRankedBarNonlinearPattern::getXmlTagName ( ) );
	std::set < alphabet::RankedSymbol > bars = TreeFromXMLParser::parseBars ( input );
	alphabet::RankedSymbol variablesBarSymbol = TreeFromXMLParser::parseRankedVariablesBar ( input );
	alphabet::RankedSymbol subtreeWildcardSymbol = TreeFromXMLParser::parseSubtreeWildcardRankedSymbol ( input );
	std::set < alphabet::RankedSymbol > nonlinearVariables = TreeFromXMLParser::parseRankedNonlinearVariables ( input );
	std::set < alphabet::RankedSymbol > rankedAlphabet = TreeFromXMLParser::parseRankedAlphabet ( input );
	std::vector < alphabet::RankedSymbol > data = TreeFromXMLParser::parseRankedContent ( input );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, PrefixRankedBarNonlinearPattern::getXmlTagName ( ) );

	return PrefixRankedBarNonlinearPattern ( std::move ( bars ), std::move ( variablesBarSymbol ), std::move ( subtreeWildcardSymbol ), std::move ( nonlinearVariables ), std::move ( rankedAlphabet ), std::move ( data ) );
}

void PrefixRankedBarNonlinearPattern::compose ( std::deque < sax::Token > & out ) const {
	out.emplace_back ( PrefixRankedBarNonlinearPattern::getXmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	TreeToXMLComposer::composeBars ( out, getBars ( ) );
	TreeToXMLComposer::composeVariablesBar ( out, getVariablesBar ( ) );
	TreeToXMLComposer::composeSubtreeWildcard ( out, getSubtreeWildcard ( ) );
	TreeToXMLComposer::composeNonlinearVariables ( out, getNonlinearVariables ( ) );
	TreeToXMLComposer::composeAlphabet ( out, getAlphabet ( ) );
	TreeToXMLComposer::composeContent ( out, m_Data );
	out.emplace_back ( PrefixRankedBarNonlinearPattern::getXmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace tree */

namespace alib {

auto prefixRankedBarNonlinearPatternParserRegister = xmlApi < tree::Tree >::ParserRegister < tree::PrefixRankedBarNonlinearPattern > ( );
auto prefixRankedBarNonlinearPatternParserRegister2 = xmlApi < tree::RankedTreeWrapper >::ParserRegister < tree::PrefixRankedBarNonlinearPattern > ( );
auto prefixRankedBarNonlinearPatternParserRegister3 = xmlApi < alib::Object >::ParserRegister < tree::PrefixRankedBarNonlinearPattern > ( );

auto PrefixRankedBarNonlinearPatternFromRankedTree = castApi::CastRegister < tree::PrefixRankedBarNonlinearPattern, tree::RankedTree > ( );
auto PrefixRankedBarNonlinearPatternFromRankedPattern = castApi::CastRegister < tree::PrefixRankedBarNonlinearPattern, tree::RankedPattern > ( );
auto PrefixRankedBarNonlinearPatternFromRankedNonlinearPattern = castApi::CastRegister < tree::PrefixRankedBarNonlinearPattern, tree::RankedNonlinearPattern > ( );
auto PrefixRankedBarNonlinearPatternFromPrefixRankedBarTree = castApi::CastRegister < tree::PrefixRankedBarNonlinearPattern, tree::PrefixRankedBarTree > ( );
auto PrefixRankedBarNonlinearPatternFromPrefixRankedBarPattern = castApi::CastRegister < tree::PrefixRankedBarNonlinearPattern, tree::PrefixRankedBarPattern > ( );
auto PrefixRankedBarNonlinearPatternCastBinder = castApi::CastPoolStringBinder < tree::PrefixRankedBarNonlinearPattern > ( tree::PrefixRankedBarNonlinearPattern::getXmlTagName ( ) );

} /* namespace alib */
