/*
 * UnrankedTree.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Stepan Plachy
 */

#include "UnrankedTree.h"

#include <iostream>
#include <algorithm>
#include <sstream>

#include "../ranked/RankedTree.h"

#include <sax/FromXMLParserHelper.h>
#include "../common/TreeFromXMLParser.h"
#include "../common/TreeToXMLComposer.h"
#include "../common/TreeAuxiliary.h"
#include "../Tree.h"
#include "../UnrankedTreeWrapper.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include <cast/CastApi.hpp>

namespace tree {

UnrankedTree::UnrankedTree ( std::set < alphabet::Symbol > alphabet, UnrankedNode tree ) : std::Components < UnrankedTree, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( alphabet ) ), std::tuple < > ( ) ), tree ( nullptr ) {
	setTree ( std::move ( tree ) );
}

UnrankedTree::UnrankedTree ( UnrankedNode pattern ) : UnrankedTree ( pattern.computeMinimalAlphabet(), pattern ) {
}

UnrankedTree::UnrankedTree ( const RankedTree & other ) : UnrankedTree ( TreeAuxiliary::unrankSymbols ( other.getAlphabet ( ) ), other.getRoot ( ).asUnranked ( ) ) {
}

UnrankedTree::UnrankedTree ( const UnrankedTree & other ) : std::Components < UnrankedTree, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( other.getAlphabet ( ) ), std::tuple < > ( ) ), tree ( other.tree ) {
	this->tree->attachAlphabet ( & ( this->getAlphabet ( ) ) );
}

UnrankedTree::UnrankedTree ( UnrankedTree && other ) noexcept : std::Components < UnrankedTree, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( other.accessComponent < GeneralAlphabet > ( ).get ( ) ) ), std::tuple < > ( ) ), tree ( std::move ( other.tree ) ) {
	this->tree->attachAlphabet ( & ( this->getAlphabet ( ) ) );
}

UnrankedTreeBase * UnrankedTree::clone ( ) const {
	return new UnrankedTree ( * this );
}

UnrankedTreeBase * UnrankedTree::plunder ( ) && {
	return new UnrankedTree ( std::move ( * this ) );
}

UnrankedTree & UnrankedTree::operator =( const UnrankedTree & other ) {
	if ( this == & other )
		return * this;

	* this = UnrankedTree ( other );

	return * this;
}

UnrankedTree & UnrankedTree::operator =( UnrankedTree && other ) noexcept {
	std::swap ( this->tree, other.tree );
	std::swap ( accessComponent < GeneralAlphabet > ( ).get ( ), other.accessComponent < GeneralAlphabet > ( ).get ( ) );

	return * this;
}

UnrankedTree::~UnrankedTree ( ) noexcept {
}

const UnrankedNode & UnrankedTree::getRoot ( ) const {
	return * tree;
}

UnrankedNode & UnrankedTree::getRoot ( ) {
	return * tree;
}

void UnrankedTree::setTree ( UnrankedNode tree ) {
	this->tree = std::make_smart < UnrankedNode > ( std::move ( tree ) );

	if ( !this->tree->attachAlphabet ( & ( this->getAlphabet ( ) ) ) )
		throw TreeException ( "Input symbols not in the alphabet." );
}

void UnrankedTree::operator >>( std::ostream & out ) const {
	out << "(UnrankedTree " << * ( this->tree ) << ")";
}

int UnrankedTree::compare ( const UnrankedTree & other ) const {
	auto first = std::tie ( * tree, getAlphabet() );
	auto second = std::tie ( * other.tree, other.getAlphabet() );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

void UnrankedTree::nicePrint ( std::ostream & os ) const {
	tree->nicePrint ( os );
}

UnrankedTree::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

UnrankedTree UnrankedTree::parse ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, UnrankedTree::getXmlTagName() );
	std::set < alphabet::Symbol > rankedAlphabet = TreeFromXMLParser::parseUnrankedAlphabet ( input );
	UnrankedNode root = alib::xmlApi < UnrankedNode >::parse ( input );
	UnrankedTree tree ( std::move ( rankedAlphabet ), std::move ( root ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, UnrankedTree::getXmlTagName() );
	return tree;
}

void UnrankedTree::compose ( std::deque < sax::Token > & out ) const {
	out.emplace_back ( UnrankedTree::getXmlTagName(), sax::Token::TokenType::START_ELEMENT );
	TreeToXMLComposer::composeAlphabet ( out, getAlphabet ( ) );
	alib::xmlApi < UnrankedNode >::compose ( out, * tree );
	out.emplace_back ( UnrankedTree::getXmlTagName(), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace tree */

namespace alib {

auto unrankedTreeParserRegister = xmlApi < tree::Tree >::ParserRegister < tree::UnrankedTree > ( );
auto unrankedTreeParserRegister2 = xmlApi < tree::UnrankedTreeWrapper >::ParserRegister < tree::UnrankedTree > ( );
auto unrankedTreeParserRegister3 = xmlApi < alib::Object >::ParserRegister < tree::UnrankedTree > ( );

auto UnrankedTreeFromRankedTree = castApi::CastRegister < tree::UnrankedTree, tree::RankedTree > ( );
auto UnrankedTreeCastBinder = castApi::CastPoolStringBinder < tree::UnrankedTree > ( tree::UnrankedTree::getXmlTagName() );

} /* namespace alib */
