#include <list>
#include "SymbolTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "alphabet/Symbol.h"
#include "alphabet/LabeledSymbol.h"
#include "alphabet/BlankSymbol.h"
#include "alphabet/StartSymbol.h"
#include "alphabet/EndSymbol.h"
#include "alphabet/RankedSymbol.h"
#include "alphabet/BarSymbol.h"

#include "factory/XmlDataFactory.hpp"

#define CPPUNIT_IMPLY( x, y )			CPPUNIT_ASSERT ( !( x ) || ( y ) )
#define CPPUNIT_EXCLUSIVE_OR( x, y )	CPPUNIT_ASSERT ( ( !( x ) && ( y ) ) || ( ( x ) && !( y ) ) )

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( SymbolTest, "alphabet" );
CPPUNIT_TEST_SUITE_REGISTRATION ( SymbolTest );

void SymbolTest::setUp ( ) {
}

void SymbolTest::tearDown ( ) {
}

void SymbolTest::testCopyConstruct ( ) {
	alphabet::Symbol symbol = alphabet::symbolFrom ( "1" );

	alphabet::Symbol symbol2 ( symbol );

	CPPUNIT_ASSERT ( symbol == symbol2 );

	alphabet::Symbol symbol3 ( std::move ( symbol ) );

	CPPUNIT_ASSERT ( symbol2 == symbol3 );
}

void SymbolTest::testXMLParser ( ) {

	alphabet::Symbol symbol = alphabet::symbolFrom ( "1" );

	{
		std::deque < sax::Token > tokens = alib::XmlDataFactory::toTokens ( symbol );
		std::string tmp;
		sax::SaxComposeInterface::printMemory ( tmp, tokens );

		std::deque < sax::Token > tokens2;
		sax::SaxParseInterface::parseMemory ( tmp, tokens2 );
		alphabet::Symbol symbol2 = alib::XmlDataFactory::fromTokens < alphabet::Symbol > ( std::move( tokens2 ) );

		CPPUNIT_ASSERT ( symbol == symbol2 );
	}
	{
		std::string tmp = alib::XmlDataFactory::toString ( symbol );
		alphabet::Symbol symbol2 = alib::XmlDataFactory::fromString < alphabet::Symbol > ( tmp );

		CPPUNIT_ASSERT ( symbol == symbol2 );
	}
}

void SymbolTest::testOrder ( ) {
	alphabet::LabeledSymbol ls ( "a" );
	alphabet::BlankSymbol bs;
	alphabet::StartSymbol ss;
	alphabet::EndSymbol es;

	alphabet::RankedSymbol rs1 ( alphabet::Symbol ( alphabet::BarSymbol { } ), 0 );
	alphabet::RankedSymbol rs2 ( alphabet::Symbol ( alphabet::BarSymbol { } ), 0 );

	CPPUNIT_EXCLUSIVE_OR ( bs < ls, ls < bs );

	CPPUNIT_ASSERT ( rs1 == rs2 );

	// CPPUNIT_ASSERT(es < ls); //End symbol typically denoted by '$' is lexicographically smaller than any other symbol
	// CPPUNIT_ASSERT(ls < ss); //Start symbol typically denoted by '^' is lexicographically bigger than any other symbol
}
