#include "PatternTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "tree/unranked/UnrankedPattern.h"
#include "tree/ranked/RankedPattern.h"
#include "tree/ranked/PrefixRankedPattern.h"
#include "tree/ranked/PrefixRankedBarPattern.h"

#include "tree/ranked/RankedNode.h"
#include "tree/unranked/UnrankedNode.h"

#include "tree/TreeException.h"

#include "factory/XmlDataFactory.hpp"

#include "alphabet/RankedSymbol.h"
#include "alphabet/SubtreeWildcardSymbol.h"
#include "alphabet/BarSymbol.h"
#include "alphabet/VariablesBarSymbol.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( PatternTest, "tree" );
CPPUNIT_TEST_SUITE_REGISTRATION( PatternTest );

void PatternTest::setUp() {
}

void PatternTest::tearDown() {
}

void PatternTest::testRankedPatternParser() {
	const alphabet::RankedSymbol a ('a', 2);
	const alphabet::RankedSymbol b ('b', 1);
	const alphabet::RankedSymbol c ('c', 0);

	const alphabet::RankedSymbol S (alphabet::Symbol { alphabet::SubtreeWildcardSymbol {} }, 0);
	const std::set<alphabet::RankedSymbol> alphabet {a, b, c, S};

	tree::RankedNode node3(c, {});
	tree::RankedNode node4(S, {});
	tree::RankedNode node2(b, {std::make_smart<tree::RankedNode>(std::move(node3))});
	tree::RankedNode node1(a, {std::make_smart<tree::RankedNode>(std::move(node2)), std::make_smart<tree::RankedNode>(std::move(node4))});

	tree::RankedPattern tree(S, alphabet, std::move(node1));

	CPPUNIT_ASSERT( tree == tree );
	tree.getRoot().nicePrint(std::cout);
	{
		std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(tree);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		std::deque<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		tree::RankedPattern tree2 = alib::XmlDataFactory::fromTokens<tree::RankedPattern>(std::move(tokens2));

		CPPUNIT_ASSERT( tree == tree2 );
		std::cout << std::endl;
		tree2.getRoot().nicePrint(std::cout);
	}

	const alphabet::Symbol ua = alphabet::symbolFrom('a');
	const alphabet::Symbol ub = alphabet::symbolFrom('b');
	const alphabet::Symbol uc = alphabet::symbolFrom('c');

	const alphabet::Symbol uS { alphabet::SubtreeWildcardSymbol {} };
	const std::set<alphabet::Symbol> ualphabet {ua, ub, uc, uS};

	tree::UnrankedNode unode3(uc, {});
	tree::UnrankedNode unode4(uS, {});
	tree::UnrankedNode unode2(ub, {std::make_smart<tree::UnrankedNode>(std::move(unode3))});
	tree::UnrankedNode unode1(ua, {std::make_smart<tree::UnrankedNode>(std::move(unode2)), std::make_smart<tree::UnrankedNode>(std::move(unode4))});

	tree::UnrankedPattern tree2(uS, ualphabet, std::move(unode1));

	tree::UnrankedPattern tree3 (tree );

	CPPUNIT_ASSERT(tree3 == tree2);
}

void PatternTest::testUnrankedPatternParser() {
	const alphabet::Symbol a = alphabet::symbolFrom('a');
	const alphabet::Symbol b = alphabet::symbolFrom('b');
	const alphabet::Symbol c = alphabet::symbolFrom('c');

	const alphabet::Symbol S { alphabet::SubtreeWildcardSymbol {} };
	const std::set<alphabet::Symbol> alphabet {a, b, c, S};

	tree::UnrankedNode node3(c, {});
	tree::UnrankedNode node4(S, {});
	tree::UnrankedNode node2(b, {std::make_smart<tree::UnrankedNode>(std::move(node3))});
	tree::UnrankedNode node1(a, {std::make_smart<tree::UnrankedNode>(std::move(node2)), std::make_smart<tree::UnrankedNode>(std::move(node4))});

	tree::UnrankedPattern tree(S, alphabet, std::move(node1));

	CPPUNIT_ASSERT( tree == tree );
	tree.getRoot().nicePrint(std::cout);
	{
		std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(tree);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		std::deque<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		tree::UnrankedPattern tree2 = alib::XmlDataFactory::fromTokens<tree::UnrankedPattern>(std::move(tokens2));

		CPPUNIT_ASSERT( tree == tree2 );
		std::cout << std::endl;
		tree2.getRoot().nicePrint(std::cout);
	}

	const alphabet::RankedSymbol ra ('a', 2);
	const alphabet::RankedSymbol rb ('b', 1);
	const alphabet::RankedSymbol rc ('c', 0);

	const alphabet::RankedSymbol rS { alphabet::Symbol { alphabet::SubtreeWildcardSymbol {} }, 0};
	const std::set<alphabet::RankedSymbol> ralphabet {ra, rb, rc, rS};

	tree::RankedNode rnode3(rc, {});
	tree::RankedNode rnode4(rS, {});
	tree::RankedNode rnode2(rb, {std::make_smart<tree::RankedNode>(std::move(rnode3))});
	tree::RankedNode rnode1(ra, {std::make_smart<tree::RankedNode>(std::move(rnode2)), std::make_smart<tree::RankedNode>(std::move(rnode4))});

	tree::RankedPattern tree2(rS, ralphabet, std::move(rnode1));

	tree::RankedPattern tree3 (tree);

	CPPUNIT_ASSERT(tree3 == tree2);
}

void PatternTest::testPrefixRankedPatternParser() {
	const alphabet::RankedSymbol a ('a', 2);
	const alphabet::RankedSymbol b ('b', 1);
	const alphabet::RankedSymbol c ('c', 0);

	const alphabet::RankedSymbol S (alphabet::Symbol { alphabet::SubtreeWildcardSymbol {} }, 0);
	const std::set<alphabet::RankedSymbol> alphabet {a, b, c, S};

	tree::RankedNode node3(c, {});
	tree::RankedNode node4(S, {});
	tree::RankedNode node2(b, {std::make_smart<tree::RankedNode>(std::move(node3))});
	tree::RankedNode node1(a, {std::make_smart<tree::RankedNode>(std::move(node2)), std::make_smart<tree::RankedNode>(std::move(node4))});

	tree::RankedPattern pattern(S, alphabet, std::move(node1));
	tree::PrefixRankedPattern pattern2(pattern);

	CPPUNIT_ASSERT( pattern2 == pattern2 );
	{
		std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(pattern2);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		std::deque<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		tree::PrefixRankedPattern pattern3 = alib::XmlDataFactory::fromTokens<tree::PrefixRankedPattern>(std::move(tokens2));

		CPPUNIT_ASSERT( pattern2 == pattern3 );
		std::cout << std::endl;
	}
}

void PatternTest::testPrefixRankedBarPatternParser() {
	const alphabet::RankedSymbol a ('a', 2);
	const alphabet::RankedSymbol b ('b', 1);
	const alphabet::RankedSymbol c ('c', 0);

	const alphabet::RankedSymbol S (alphabet::Symbol { alphabet::SubtreeWildcardSymbol {} }, 0);
	const std::set<alphabet::RankedSymbol> alphabet {a, b, c, S};

	tree::RankedNode node3(c, {});
	tree::RankedNode node4(S, {});
	tree::RankedNode node2(b, {std::make_smart<tree::RankedNode>(std::move(node3))});
	tree::RankedNode node1(a, {std::make_smart<tree::RankedNode>(std::move(node2)), std::make_smart<tree::RankedNode>(std::move(node4))});

	tree::RankedPattern pattern(S, alphabet, std::move(node1));
	tree::PrefixRankedBarPattern pattern2(alphabet::BarSymbol::BAR_SYMBOL, alphabet::VariablesBarSymbol::RANKED_BAR_SYMBOL, pattern);

	CPPUNIT_ASSERT( pattern2 == pattern2 );
	{
		std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(pattern2);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		std::cout << tmp << std::endl << std::endl;

		std::deque<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		tree::PrefixRankedBarPattern pattern3 = alib::XmlDataFactory::fromTokens<tree::PrefixRankedBarPattern>(std::move(tokens2));

		CPPUNIT_ASSERT( pattern2 == pattern3 );
		std::cout << std::endl;
	}
}

