#include "VariantTest.h"
#include <variant>
#include <set>
#include <algorithm>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( VariantTest, "bits" );
CPPUNIT_TEST_SUITE_REGISTRATION( VariantTest );

void VariantTest::setUp() {
}

void VariantTest::tearDown() {
}

void VariantTest::testVariant() {
	std::variant<int, std::string, VariantTest::test> d(10);
	CPPUNIT_ASSERT( d.is<int>() );
	CPPUNIT_ASSERT( d.get<int>() == 10 );

	std::string str = "abcde";
	d.set<std::string>(str);
	CPPUNIT_ASSERT( d.is<std::string>() );
	CPPUNIT_ASSERT( d.get<std::string>() == str );

	d.set<VariantTest::test>(VariantTest::test());
	*(d.get<VariantTest::test>().holder) = 42;

	VariantTest::test tmp;
	*(tmp.holder) = 42;
	CPPUNIT_ASSERT( d.is<VariantTest::test>() );
	CPPUNIT_ASSERT( d.get<VariantTest::test>() == tmp );

	std::variant<int, std::string, VariantTest::test> e(d);
	CPPUNIT_ASSERT( e.get<VariantTest::test>() == tmp );

	*(e.get<VariantTest::test>().holder) = 43;

	CPPUNIT_ASSERT( (d < e) == true );
	CPPUNIT_ASSERT( (e < d) == false );

	CPPUNIT_ASSERT( (e == d) == false );

	d = std::variant<int, std::string, VariantTest::test>(0);
	d = e;

	CPPUNIT_ASSERT( *(d.get<VariantTest::test>().holder) == 43 );

	CPPUNIT_ASSERT( (d < e) == false );
	CPPUNIT_ASSERT( (e < d) == false );

	CPPUNIT_ASSERT( (e == d) == true );
}

void VariantTest::testVariantSet() {
	VariantTest::test tmp;
	*(tmp.holder) = 42;

	std::string str = "abcde";

	std::variant<int, std::string, VariantTest::test> d(VariantTest::test {});
	*(d.get<VariantTest::test>().holder) = 42;
	std::variant<int, std::string, VariantTest::test> e(d);

	std::variant<int, std::string, VariantTest::test> f(5);

	std::variant<int, std::string, VariantTest::test> g(str);

	std::set<std::variant<int, std::string, VariantTest::test>> testSet;
	testSet.insert(d);
	testSet.insert(e);
	testSet.insert(f);
	testSet.insert(g);

	CPPUNIT_ASSERT( testSet.size() == 3 );

	for(std::set<std::variant<int, std::string, VariantTest::test>>::const_iterator iter = testSet.begin(); iter != testSet.end(); iter++) {
		if(iter->is<int>())
			CPPUNIT_ASSERT( iter->get<int>() == 5);
		if(iter->is<std::string>())
			CPPUNIT_ASSERT( iter->get<std::string>() == str );
		if(iter->is<VariantTest::test>())
			CPPUNIT_ASSERT( iter->get<VariantTest::test>() == tmp );
	}

	std::variant<VariantTest::test2, std::string, VariantTest::test> h("aa");
	CPPUNIT_ASSERT( h.is<std::string>() );

	std::map<std::variant<std::string, int>, int> testMap;
	testMap.insert(std::make_pair(std::variant<std::string, int> {"aa"}, 10));

	CPPUNIT_ASSERT( testMap.size() == 1 );
	CPPUNIT_ASSERT( testMap.find(std::variant<std::string, int> {"aa"}) != testMap.end() );
	CPPUNIT_ASSERT( testMap.find(std::variant<std::string, int> {10}) == testMap.end() );

	testMap.insert(std::make_pair(std::variant<std::string, int> {"ab"}, 11));
	testMap.insert(std::make_pair(std::variant<std::string, int> {3}, 13));

	CPPUNIT_ASSERT( testMap.find(std::variant<std::string, int> {"aa"}) != testMap.end() );
	CPPUNIT_ASSERT( testMap.find(std::variant<std::string, int> {10}) == testMap.end() );
}

void VariantTest::testVariantSet2() {
	std::set<std::variant<std::string, int>> s, t, u;
	s.insert(std::string { "aa" } );
	s.insert(123);

	std::set_union(s.begin(), s.end(), t.begin(), t.end(), std::inserter(u, u.begin()));

	CPPUNIT_ASSERT( s.size() == u.size());
}
