/*
 * RegExpFromStringParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef REG_EXP_FROM_STRING_PARSER_H_
#define REG_EXP_FROM_STRING_PARSER_H_

#include "regexp/RegExpFeatures.h"
#include "RegExpFromStringLexer.h"

#include <set>
#include <memory>

namespace alib {

template<typename T>
struct stringApi;

} /* namespace alib */

namespace regexp {

class RegExpFromStringParser {
public:
	RegExpFromStringParser() {}

private:
	std::rvalue_ref < UnboundedRegExpElement > alternation(std::istream& input) const;
	std::rvalue_ref < UnboundedRegExpElement > alternationCont(std::istream& input, std::rvalue_ref < UnboundedRegExpElement > left) const;
	std::rvalue_ref < UnboundedRegExpElement > alternationContCont(std::istream& input, UnboundedRegExpAlternation left) const;

	std::rvalue_ref < UnboundedRegExpElement > concatenation(std::istream& input) const;
	std::rvalue_ref < UnboundedRegExpElement > concatenationCont(std::istream& input, std::rvalue_ref < UnboundedRegExpElement > left) const;
	std::rvalue_ref < UnboundedRegExpElement > concatenationContCont(std::istream& input, UnboundedRegExpConcatenation left) const;

	std::rvalue_ref < UnboundedRegExpElement > factor(std::istream& input) const;
	std::rvalue_ref < UnboundedRegExpElement > star(std::istream& input, UnboundedRegExpElement && elem) const;

	RegExpFromStringLexer m_RegexpLexer;

	RegExp parseRegExp(std::istream& input) const;
	RegExp parseRegExp(std::istream& input, const std::set<FEATURES>& features) const;

	template<typename T> friend struct alib::stringApi;
};

} /* namespace regexp */

#endif /* REG_EXP_FROM_STRING_PARSER_H_ */
