/*
 * RegExpStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "RegExpStat.h"
#include "common/Printer.hpp"

#include <exception/CommonException.h>

#include <regexp/formal/FormalRegExp.h>
#include <regexp/formal/FormalRegExpElement.h>
#include <regexp/unbounded/UnboundedRegExp.h>
#include <regexp/unbounded/UnboundedRegExpElement.h>

void RegExpStat::stat ( const regexp::RegExp & regexp, const RegExpSettings & settings ) {
	dispatch ( regexp.getData ( ), settings );
}

unsigned RegExpStat::countNodes ( const regexp::FormalRegExpElement & ) {
	throw exception::CommonException ( "unimplemented" );
}

void RegExpStat::stat ( const regexp::FormalRegExp & regexp, const RegExpSettings & settings ) {
	Printer::PrintSet ( settings.alphabet, regexp.getAlphabet ( ) );

	switch ( settings.nodes ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( regexp.getRegExp ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << countNodes ( regexp.getRegExp ( ) ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( regexp.getRegExp ( ) );
		std::cout << countNodes ( regexp.getRegExp ( ) ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto RegExpStatFormalRegExp = RegExpStat::RegistratorWrapper < void, regexp::FormalRegExp > ( RegExpStat::stat );

unsigned RegExpStat::countNodes ( const regexp::UnboundedRegExpElement & ) {
	throw exception::CommonException ( "unimplemented" );
}

void RegExpStat::stat ( const regexp::UnboundedRegExp & regexp, const RegExpSettings & settings ) {
	Printer::PrintSet ( settings.alphabet, regexp.getAlphabet ( ) );

	switch ( settings.nodes ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( regexp.getRegExp ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << countNodes ( regexp.getRegExp ( ) ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( regexp.getRegExp ( ) );
		std::cout << countNodes ( regexp.getRegExp ( ) ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto RegExpStatUnboundedRegExp = RegExpStat::RegistratorWrapper < void, regexp::UnboundedRegExp > ( RegExpStat::stat );
