/*
 * TreeStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "TreeStat.h"
#include "common/Printer.hpp"

#include <tree/ranked/RankedTree.h>
#include <tree/unranked/UnrankedTree.h>

void TreeStat::stat ( const tree::Tree & tree, const TreeSettings & settings ) {
	dispatch ( tree.getData ( ), settings );
}

unsigned TreeStat::countNodes ( const tree::RankedNode & node ) {
	unsigned res = 1;

	for ( const std::smart_ptr < const tree::RankedNode > & child : node.getChildren ( ) )
		res += countNodes ( * child );

	return res;
}

void TreeStat::stat ( const tree::RankedTree & tree, const TreeSettings & settings ) {
	Printer::PrintSet ( settings.alphabet, tree.getAlphabet ( ) );

	switch ( settings.nodes ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( tree.getRoot ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << countNodes ( tree.getRoot ( ) ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( tree.getRoot ( ) );
		std::cout << countNodes ( tree.getRoot ( ) ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto TreeStatFormalTree = TreeStat::RegistratorWrapper < void, tree::RankedTree > ( TreeStat::stat );

unsigned TreeStat::countNodes ( const tree::UnrankedNode & node ) {
	unsigned res = 1;

	for ( const std::smart_ptr < const tree::UnrankedNode > & child : node.getChildren ( ) )
		res += countNodes ( * child );

	return res;
}

void TreeStat::stat ( const tree::UnrankedTree & tree, const TreeSettings & settings ) {
	Printer::PrintSet ( settings.alphabet, tree.getAlphabet ( ) );

	switch ( settings.nodes ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( tree.getRoot ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << countNodes ( tree.getRoot ( ) ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( tree.getRoot ( ) );
		std::cout << countNodes ( tree.getRoot ( ) ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto TreeStatUnboundedTree = TreeStat::RegistratorWrapper < void, tree::UnrankedTree > ( TreeStat::stat );
