/*
 * AutomataStat.h
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#ifndef PRINTER_H_
#define PRINTER_H_

#include "PrintingOptions.h"

#include <factory/XmlDataFactory.hpp>
#include <container/ObjectsSet.h>
#include <container/ObjectsMap.h>
#include <container/ObjectsTuple.h>
#include <container/ObjectsVector.h>
#include <container/ObjectsVariant.h>
#include <container/ObjectsPair.h>

#include <iostream>

class Printer {
public:
	template < class ... T >
	static void PrintSet ( PrintingOptions printingOptions, const std::set < T ... > & data ) {
		switch ( printingOptions ) {
		case PrintingOptions::PRINT:
			alib::XmlDataFactory::toStdout ( data );
			break;

		case PrintingOptions::SIZE:
			std::cout << data.size ( ) << std::endl;
			break;

		case PrintingOptions::BOTH:
			std::cout << data.size ( ) << std::endl;
			alib::XmlDataFactory::toStdout ( data );
			break;

		case PrintingOptions::NOOP:
			break;
		}
	}

	template < class ... T >
	static void PrintVector ( PrintingOptions printingOptions, const std::vector < T ... > & data ) {
		switch ( printingOptions ) {
		case PrintingOptions::PRINT:
			alib::XmlDataFactory::toStdout ( data );
			break;

		case PrintingOptions::SIZE:
			std::cout << data.size ( ) << std::endl;
			break;

		case PrintingOptions::BOTH:
			std::cout << data.size ( ) << std::endl;
			alib::XmlDataFactory::toStdout ( data );
			break;

		case PrintingOptions::NOOP:
			break;
		}
	}

	template < class ... T >
	static void PrintMap ( PrintingOptions printingOptions, const std::map < T ... > & data ) {
		switch ( printingOptions ) {
		case PrintingOptions::PRINT:
			alib::XmlDataFactory::toStdout ( data );
			break;

		case PrintingOptions::SIZE:
			std::cout << data.size ( ) << std::endl;
			break;

		case PrintingOptions::BOTH:
			std::cout << data.size ( ) << std::endl;
			alib::XmlDataFactory::toStdout ( data );
			break;

		case PrintingOptions::NOOP:
			break;
		}
	}

	template < class T >
	static void PrintValue ( PrintingOptions printingOptions, const T & data ) {
		switch ( printingOptions ) {
		case PrintingOptions::PRINT:
			alib::XmlDataFactory::toStdout ( data );
			break;

		case PrintingOptions::SIZE:
			std::cout << 1 << std::endl;
			break;

		case PrintingOptions::BOTH:
			std::cout << 1 << std::endl;
			alib::XmlDataFactory::toStdout ( data );
			break;

		case PrintingOptions::NOOP:
			break;
		}
	}

};

#endif // PRINTER_H_
